@@ -1,20 +1,22 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model;
-$Config::Model::VERSION = '2.061';
+$Config::Model::VERSION = '2.082';
+use strict ;
+use warnings;
+use 5.10.1;
+
 use Mouse;
 use Mouse::Util::TypeConstraints;
 use MouseX::StrictConstructor;
 
-use 5.10.1;
-
 use Carp;
 use Storable ('dclone');
 use Data::Dumper ();
@@ -26,6 +28,9 @@ use File::Path qw/make_path/;
 # this class holds the version number of the package
 use vars qw(@status @level %default_property);
 
+my $legacy_logger = get_logger("Model::Legacy") ;
+my $logger = get_logger("Model") ;
+
 %default_property = (
     status      => 'standard',
     level       => 'normal',
@@ -87,7 +92,7 @@ has normalized_models => (
 # This attribute contain the model that will be used by Config::Model::Node. They
 # are created on demand when get_model is called. When created the inclusion of
 # other classes is done according to the class 'include' parameter. Note that get_model
-# will try call load if the required normalized_model is not known (lazy loading)
+# will try to call load if the required normalized_model is not known (lazy loading)
 has models => (
     isa     => 'HashRef',
     is      => 'ro',
@@ -197,7 +202,7 @@ sub instance {
 
 sub instance_names {
     my $self = shift;
-    return keys %{ $self->instances };
+    return sort keys %{ $self->instances };
 }
 
 @level = qw/hidden normal important/;
@@ -215,11 +220,17 @@ sub instance_names {
 # description, summary, level, status are moved
 # into element description.
 
-my @legal_params = qw/element status description summary level
-    config_dir
-    read_config read_config_dir write_config
-    write_config_dir accept/;
-my @static_legal_params = qw/class_description copyright author license generated_by/;
+my @legal_params_to_move = (
+    qw/read_config  read_config_dir
+       write_config write_config_dir/,    # read/write stuff
+
+    # this parameter is filled by class generated by a program. It may
+    # be used to avoid interactive edition of a generated model
+    'generated_by',
+    qw/class_description author copyright license include include_after include_backend class/
+);
+
+my @other_legal_params = qw/ author element status description summary level accept/;
 
 # keep as external API. All internal call go through _store_model
 #  See comments around raw_models attribute for explanations
@@ -253,7 +264,7 @@ sub merge_included_class {
     my $normalized_model = $self->normalized_model($config_class_name);
     my $model            = dclone $normalized_model ;
 
-    # add included items
+    # add included elements
     if ( $self->skip_include and defined $normalized_model->{include} ) {
         my $inc = $normalized_model->{include};
         $model->{include} = ref $inc ? $inc : [$inc];
@@ -264,9 +275,47 @@ sub merge_included_class {
         # include class in raw_copy, normalized_model is left as is
         $self->include_class( $config_class_name, $model );
     }
+
+    # add included backend
+    if ( $self->skip_include and defined $normalized_model->{include_backend} ) {
+        my $inc = $normalized_model->{include_backend};
+        $model->{include_backend} = ref $inc ? $inc : [$inc];
+    }
+    else {
+        # include read/write config specifications in raw_copy,
+        # normalized_model is left as is
+        $self->include_backend( $config_class_name, $model );
+    }
+
     return $model;
 }
 
+sub include_backend {
+    my $self         = shift;
+    my $class_name   = shift || croak "include_backend: undef includer";
+    my $target_model = shift || die "include_backend:: undefined target_model";
+
+    my $included_classes = delete $target_model->{include_backend};
+    return () unless defined $included_classes;
+
+    foreach my $included_class (@$included_classes) {
+        # takes care of recursive include, because get_model will perform
+        # includes (and normalization). Is already a dclone
+        my $included_model = $self->get_model($included_class);
+
+        foreach my $rw (qw/read_config write_config config_dir/) {
+            if ($target_model->{$rw} and $included_model->{$rw}) {
+                my $msg = "Included $rw from $included_class cannot clobber "
+                    . "existing data in $class_name";
+                Config::Model::Exception::ModelDeclaration->throw( error => $msg );
+            }
+            elsif ($included_model->{$rw}) {
+                $target_model->{$rw} = $included_model->{$rw};
+            }
+        }
+    }
+}
+
 sub normalize_class_parameters {
     my $self              = shift;
     my $config_class_name = shift || die;
@@ -326,17 +375,7 @@ sub normalize_class_parameters {
     $model->{read_config_dir} = $model->{write_config_dir} = delete $normalized_model->{config_dir}
         if defined $normalized_model->{config_dir};
 
-    my @info_to_move = (
-        qw/read_config  read_config_dir
-            write_config write_config_dir/,    # read/write stuff
-
-        # this parameter is filled by class generated by a program. It may
-        # be used to avoid interactive edition of a generated model
-        'generated_by',
-        qw/class_description author copyright license include include_after/
-    );
-
-    foreach my $info (@info_to_move) {
+    foreach my $info (@legal_params_to_move) {
         next unless defined $normalized_model->{$info};
         $model->{$info} = delete $normalized_model->{$info};
     }
@@ -425,9 +464,9 @@ sub normalize_class_parameters {
     Config::Model::Exception::ModelDeclaration->throw(
               error => "create class $config_class_name: unexpected "
             . "parameters '"
-            . join( ', ', keys %$normalized_model ) . "' "
+            . join( ', ', sort keys %$normalized_model ) . "' "
             . "Expected '"
-            . join( "', '", @legal_params, @static_legal_params )
+            . join( "', '", @legal_params_to_move, @other_legal_params )
             . "'" )
         if keys %$normalized_model;
 
@@ -522,6 +561,7 @@ sub translate_legacy_info {
                 $self->translate_id_default_info( $config_class_name, $elt_name, $rule_effect );
             }
         }
+        $self->translate_id_class($config_class_name, $elt_name, $info );
     }
 
     if ( defined $info->{type} and ( $info->{type} eq 'leaf' ) ) {
@@ -532,7 +572,9 @@ sub translate_legacy_info {
         $self->translate_legacy_built_in_list( $config_class_name, $info, $info, );
     }
 
-    print Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ), "\n" if $::debug;
+    $legacy_logger->debug(
+        Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ) 
+    ) if $legacy_logger->is_debug;
 }
 
 sub translate_cargo_info {
@@ -564,7 +606,9 @@ sub translate_cargo_info {
     }
 
     $info->{cargo} = \%cargo;
-    print Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ), "\n" if $::debug;
+    $legacy_logger->debug( 
+        Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ) 
+    ) if $legacy_logger->is_debug;
 }
 
 sub translate_id_names {
@@ -616,9 +660,10 @@ sub translate_compute_info {
 
     if ( ref( $info->{$old_name} ) eq 'ARRAY' ) {
         my $compute_info = delete $info->{$old_name};
-        print "translate_compute_info $elt_name input:\n",
-            Data::Dumper->Dump( [$compute_info], [qw/compute_info/] ), "\n"
-            if $::debug;
+        $legacy_logger->debug(
+            "translate_compute_info $elt_name input:\n",
+            Data::Dumper->Dump( [$compute_info], [qw/compute_info/] )
+        ) if $legacy_logger->is_debug;
 
         $self->show_legacy_issue( "$config_class_name->$elt_name: specifying compute info with ",
             "an array ref is deprecated" );
@@ -640,12 +685,41 @@ sub translate_compute_info {
         };
         $info->{$new_name}{replace} = $replace_h if defined $replace_h;
 
-        print "translate_warp_info $elt_name output:\n",
-            Data::Dumper->Dump( [ $info->{$new_name} ], [ 'new_' . $new_name ] ), "\n"
-            if $::debug;
+        $legacy_logger->debug(
+            "translate_warp_info $elt_name output:\n",
+            Data::Dumper->Dump( [ $info->{$new_name} ], [ 'new_' . $new_name ] )
+        ) if $legacy_logger->is_debug;
     }
 }
 
+sub translate_id_class {
+    my $self              = shift;
+    my $config_class_name = shift || die;
+    my $elt_name          = shift;
+    my $info              = shift;
+
+
+    $legacy_logger->debug(
+        "translate_id_class $elt_name input:\n",
+        Data::Dumper->Dump( [$info], [qw/info/] )
+    ) if $legacy_logger->is_debug;
+
+    my $class_overide_param = $info->{type}.'_class';
+    my $class_overide = $info->{$class_overide_param};
+    if ($class_overide) {
+        $info->{class} = $class_overide;
+        $self->show_legacy_issue(
+            "$config_class_name->$elt_name: '$class_overide_param' is deprecated, ",
+            "Use 'class' instead."
+        );
+    }
+
+    $legacy_logger->debug(
+        "translate_id_class $elt_name output:",
+        Data::Dumper->Dump( [$info], [qw/new_info/])
+    ) if $legacy_logger->is_debug;
+}
+
 # internal: translate default information for id element
 sub translate_id_default_info {
     my $self              = shift;
@@ -653,9 +727,10 @@ sub translate_id_default_info {
     my $elt_name          = shift;
     my $info              = shift;
 
-    print "translate_id_default_info $elt_name input:\n",
-        Data::Dumper->Dump( [$info], [qw/info/] ), "\n"
-        if $::debug;
+    $legacy_logger->debug(
+        "translate_id_default_info $elt_name input:\n",
+        Data::Dumper->Dump( [$info], [qw/info/] )
+    ) if $legacy_logger->is_debug;
 
     my $warn = "$config_class_name->$elt_name: 'default' parameter for list or "
         . "hash element is deprecated. ";
@@ -673,9 +748,11 @@ sub translate_id_default_info {
         $info->{default_keys} = [$def_info];
         $self->show_legacy_issue( $warn, "Use default_keys" );
     }
-    print "translate_id_default_info $elt_name output:\n",
-        Data::Dumper->Dump( [$info], [qw/new_info/] ), "\n"
-        if $::debug;
+
+    $legacy_logger->debug( 
+        "translate_id_default_info $elt_name output:",
+        Data::Dumper->Dump( [$info], [qw/new_info/])
+    ) if $legacy_logger->is_debug;
 }
 
 # internal: translate auto_create information for id element
@@ -685,9 +762,10 @@ sub translate_id_auto_create {
     my $elt_name          = shift;
     my $info              = shift;
 
-    print "translate_id_auto_create $elt_name input:\n",
-        Data::Dumper->Dump( [$info], [qw/info/] ), "\n"
-        if $::debug;
+    $legacy_logger->debug(
+        "translate_id_auto_create $elt_name input:",
+        Data::Dumper->Dump( [$info], [qw/info/] )
+    ) if $legacy_logger->is_debug;
 
     my $warn = "$config_class_name->$elt_name: 'auto_create' parameter for list or "
         . "hash element is deprecated. ";
@@ -706,9 +784,10 @@ sub translate_id_auto_create {
         die "Unexpected element ($elt_name) type $info->{type} ", "for translate_id_auto_create";
     }
 
-    print "translate_id_default_info $elt_name output:\n",
-        Data::Dumper->Dump( [$info], [qw/new_info/] ), "\n"
-        if $::debug;
+    $legacy_logger->debug(
+        "translate_id_default_info $elt_name output:\n",
+        Data::Dumper->Dump( [$info], [qw/new_info/] )
+    ) if $legacy_logger->is_debug;
 }
 
 sub translate_id_min_max {
@@ -720,8 +799,8 @@ sub translate_id_min_max {
     foreach my $bad (qw/min max/) {
         next unless defined $info->{$bad};
 
-        print "translate_id_min_max $elt_name $bad:\n"
-            if $::debug;
+        $legacy_logger->debug( "translate_id_min_max $elt_name $bad:")
+            if $legacy_logger->is_debug;
 
         my $good = $bad . '_index';
         my $warn = "$config_class_name->$elt_name: '$bad' parameter for list or "
@@ -735,9 +814,10 @@ sub translate_id_min_max {
 sub translate_warp_info {
     my ( $self, $config_class_name, $elt_name, $type, $warp_info ) = @_;
 
-    print "translate_warp_info $elt_name input:\n",
-        Data::Dumper->Dump( [$warp_info], [qw/warp_info/] ), "\n"
-        if $::debug;
+    $legacy_logger->debug(
+        "translate_warp_info $elt_name input:\n",
+        Data::Dumper->Dump( [$warp_info], [qw/warp_info/] )
+    ) if $legacy_logger->is_debug;
 
     my $follow = $self->translate_follow_arg( $config_class_name, $elt_name, $warp_info->{follow} );
 
@@ -753,9 +833,10 @@ sub translate_warp_info {
     $warp_info->{follow} = $follow;
     $warp_info->{rules}  = $rules;
 
-    print "translate_warp_info $elt_name output:\n",
-        Data::Dumper->Dump( [$warp_info], [qw/new_warp_info/] ), "\n"
-        if $::debug;
+    $legacy_logger->debug(
+        "translate_warp_info $elt_name output:\n",
+        Data::Dumper->Dump( [$warp_info], [qw/new_warp_info/] )
+    ) if $legacy_logger->is_debug;
 }
 
 # internal
@@ -898,16 +979,17 @@ sub translate_legacy_builtin {
     my $raw_builtin_default = delete $normalized_model->{built_in};
     return unless defined $raw_builtin_default;
 
-    print Data::Dumper->Dump( [$normalized_model], ['builtin to translate'] ), "\n"
-        if $::debug;
+    $legacy_logger->debug( 
+        Data::Dumper->Dump( [$normalized_model], ['builtin to translate'] )
+    ) if $legacy_logger->is_debug;
 
     $self->show_legacy_issue( "$config_class_name: parameter 'built_in' is deprecated "
             . "in favor of 'upstream_default'" );
 
     $model->{upstream_default} = $raw_builtin_default;
 
-    print Data::Dumper->Dump( [$model], ['translated_builtin'] ), "\n"
-        if $::debug;
+    $legacy_logger->debug( Data::Dumper->Dump( [$model], ['translated_builtin'] )) 
+        if $legacy_logger->is_debug;
 }
 
 sub translate_legacy_built_in_list {
@@ -916,16 +998,17 @@ sub translate_legacy_built_in_list {
     my $raw_builtin_default = delete $normalized_model->{built_in_list};
     return unless defined $raw_builtin_default;
 
-    print Data::Dumper->Dump( [$normalized_model], ['built_in_list to translate'] ), "\n"
-        if $::debug;
+    $legacy_logger->debug( 
+        Data::Dumper->Dump( [$normalized_model], ['built_in_list to translate'] )
+    ) if $legacy_logger->is_debug;
 
     $self->show_legacy_issue( "$config_class_name: parameter 'built_in_list' is deprecated "
             . "in favor of 'upstream_default_list'" );
 
     $model->{upstream_default_list} = $raw_builtin_default;
 
-    print Data::Dumper->Dump( [$model], ['translated_built_in_list'] ), "\n"
-        if $::debug;
+    $legacy_logger->debug( Data::Dumper->Dump( [$model], ['translated_built_in_list'] ))
+        if $legacy_logger->is_debug;
 }
 
 sub include_class {
@@ -982,7 +1065,7 @@ sub include_one_class {
             my $msg =
                   "Unknown element for 'include_after': "
                 . "$include_after, expected "
-                . join( ' ', keys %elt_idx );
+                . join( ' ', sort keys %elt_idx );
             Config::Model::Exception::ModelDeclaration->throw( error => $msg );
         }
 
@@ -1139,9 +1222,9 @@ sub augment_config_class_really {
     foreach my $list_name (qw/element_list accept_list/) {
         my %seen;
         my @newlist;
-        foreach ( @{ $new_model->{$list_name} } ) {
-            push @newlist, $_ unless $seen{$_};
-            $seen{$_} = 1;
+        foreach my $elt ( @{ $new_model->{$list_name} } ) {
+            push @newlist, $elt unless $seen{$elt};
+            $seen{$elt} = 1;
         }
 
         $new_model->{$list_name} = \@newlist;
@@ -1279,10 +1362,10 @@ sub get_migrate_doc {
     else                     { $mform = "'C<$mform>' " }
 
     my $mdoc = "Note: $elt_name $desc ${mform}and with "
-        . join( ", ", map { qq!\$$_ => "C<$mv->{$_}>"! } keys %$mv );
+        . join( ", ", map { qq!\$$_ => "C<$mv->{$_}>"! } sort keys %$mv );
     if ( my $rep = $migr->{replace} ) {
         $mdoc .=
-            ' and ' . join( ", ", map { qq!'C<\$replace{$_}>' => "C<$rep->{$_}>"! } keys %$rep );
+            ' and ' . join( ", ", map { qq!'C<\$replace{$_}>' => "C<$rep->{$_}>"! } sort keys %$rep );
     }
 
     return ( $mdoc, '' );
@@ -1349,7 +1432,7 @@ sub generate_doc {
     my @wrote;
 
     if ( defined $dir and $dir ) {
-        foreach my $class_name ( keys %$res ) {
+        foreach my $class_name ( sort keys %$res ) {
             my $file = $class_name;
             $file =~ s!::!/!g;
             my $pl_file  = $dir . "/$file.pl";
@@ -1376,7 +1459,7 @@ sub generate_doc {
         }
     }
     else {
-        foreach my $class_name ( keys %$res ) {
+        foreach my $class_name ( sort keys %$res ) {
             print "########## $class_name ############ \n\n";
             print $res->{$class_name};
         }
@@ -1455,8 +1538,8 @@ sub get_element_property {
     my $model = $self->model($class);
 
     # must take into account 'accept' model parameter
-    if ( not defined $model->{element}{$prop} ) {
-
+    if ( not defined $model->{element}{$elt} ) {
+        $logger->debug("test accept for class $class elt $elt prop $prop");
         foreach my $acc_re ( @{ $model->{accept_list} } ) {
             return $model->{accept}{$acc_re}{$prop} || $default_property{$prop}
                 if $elt =~ /^$acc_re$/;
@@ -1514,7 +1597,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
@@ -1528,8 +1611,7 @@ version 2.061
  my $model = Config::Model->new() ; # Config::Model object
 
  # create config model. Most users will want to store the model
- # in lib/Config/Model/models and run "config-edit -model MiniModel"
- # See below for details
+ # in lib/Config/Model/models and run cme as explained below
  $model ->create_config_class (
    name => "MiniModel",
    element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ],
@@ -1555,14 +1637,14 @@ version 2.061
 =head2 More convenient
 
  $ mkdir -p lib/Config/Model/models/
- $ echo "[ { name => 'MiniModel',
-             element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ],
-             read_config => { backend => 'IniFile', auto_create => 1,
-                              config_dir => '.', file => 'mini.ini',
-                            }
-           }
+ $ echo "[ { name => 'MiniModel', \
+             element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ], \
+             read_config => { backend => 'IniFile', auto_create => 1, \
+                              config_dir => '.', file => 'mini.ini', \
+                            } \
+           } \
          ] ; " > lib/Config/Model/models/MiniModel.pl
- $ config-edit -model MiniModel -model_dir lib/Config/Model/models/ -ui none bar=BARV foo=FOOV baz=BAZV
+ $ cme modify -try MiniModel -dev bar=BARV foo=FOOV baz=BAZV
  $ cat mini.ini
 
 =head2 Look Ma, no Perl
@@ -1574,7 +1656,7 @@ version 2.061
                    element:bar type=leaf value_type=uniline - \
                    element:baz type=leaf value_type=uniline - \
    read_config:0 backend=IniFile file=mini.ini config_dir=. auto_create=1 - - -
- $ config-edit -model MiniModel -model_dir lib/Config/Model/models/ -ui none bar=BARV foo=FOOV baz=BAZV
+ $ cme modify -try MiniModel -dev bar=BARV foo=FOOV baz=BAZV
  $ cat mini.ini
 
 =head1 DESCRIPTION
@@ -2129,8 +2211,22 @@ Now the element of your class will be:
 
   ( bar , foo , xyz , baz )
 
+Note that include may not clobber an existing element.
+
+=item include_backend
+
+Include read/write specification from another class.
+
+  include_backend => 'AnotherClass' ,
+
+or
+
+  include_backend => [qw/ClassOne ClassTwo/]
+
 =back
 
+Note that include may not clobber an existing read/write specification.
+
 =head2 create_config_class
 
 This method creates configuration classes. The parameters are
@@ -2220,8 +2316,9 @@ Generate POD document for configuration class.
 
 =head2 generate_doc ( top_class_name , [ directory ] )
 
-Generate POD document for configuration class top_class_name
-and write them on STDOUT or in specified directory.
+Generate POD document for configuration class top_class_name and all
+classes used by top_class_name, and write them on STDOUT or in
+specified directory.
 
 Returns a list of written file names.
 
@@ -2257,8 +2354,7 @@ debugging your configuration model.
 
 =head1 Error handling
 
-Errors are handled with an exception mechanism (See
-L<Exception::Class>).
+Errors are handled with an exception mechanism.
 
 When a strongly typed Value object gets an authorized value, it raises
 an exception. If this exception is not caught, the programs exits.
@@ -2268,14 +2364,13 @@ the various exception classes provided with C<Config::Model>.
 
 =head1 Logging
 
-See L<config-edit/Logging>
+See L<cme/Logging>
 
 =head1 BUGS
 
 Given Murphy's law, the author is fairly confident that you will find
-bugs or miss some features. Please report them to config-model at
-rt.cpan.org, or through the web interface at
-https://rt.cpan.org/Public/Bug/Report.html?Queue=config-model .
+bugs or miss some features. Please report them to
+https://github.com/dod38fr/config-model/issues
 The author will be notified, and then you'll automatically be
 notified of progress on your bug.
 
@@ -2352,7 +2447,7 @@ L<Config::Model::WarpedNode> <- L<Config::Model::AnyThing>
 
 =head2 command line
 
-L<cme>. L<config-edit> is now deprecated.
+L<cme>.
 
 =head2 Read and write backends
 
@@ -2470,7 +2565,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
@@ -2489,14 +2584,6 @@ in addition to those websites please use your favorite search engine to discover
 
 =item *
 
-MetaCPAN
-
-A modern, open-source CPAN search engine, useful to view POD in HTML format.
-
-L<http://metacpan.org/release/Config-Model>
-
-=item *
-
 Search CPAN
 
 The default CPAN search engine, useful to view POD in HTML format.
@@ -2505,14 +2592,6 @@ L<http://search.cpan.org/dist/Config-Model>
 
 =item *
 
-RT: CPAN's Bug Tracker
-
-The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Config-Model>
-
-=item *
-
 AnnoCPAN
 
 The AnnoCPAN is a website that allows community annotations of Perl module documentation.
@@ -2529,14 +2608,6 @@ L<http://cpanratings.perl.org/d/Config-Model>
 
 =item *
 
-CPAN Forum
-
-The CPAN Forum is a web forum for discussing Perl modules.
-
-L<http://cpanforum.com/dist/Config-Model>
-
-=item *
-
 CPANTS
 
 The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
@@ -2571,8 +2642,8 @@ L<http://deps.cpantesters.org/?module=Config::Model>
 
 =head2 Bugs / Feature Requests
 
-Please report any bugs or feature requests by email to C<bug-config-model at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Config-Model>. You will be automatically notified of any
+Please report any bugs or feature requests by email to C<ddumont at cpan.org>, or through
+the web interface at L<https://github.com/dod38fr/config-model/issues>. You will be automatically notified of any
 progress on the request by the system.
 
 =head2 Source Code
