@@ -1,15 +1,18 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyId;
-$Config::Model::AnyId::VERSION = '2.061';
+$Config::Model::AnyId::VERSION = '2.082';
+use 5.010;
+
 use Mouse;
+with "Config::Model::Role::NodeLoader";
 
 use Config::Model::Exception;
 use Config::Model::Warper;
@@ -21,6 +24,7 @@ use Mouse::Util::TypeConstraints;
 extends qw/Config::Model::AnyThing/;
 
 my $logger = get_logger("Tree::Element::Id");
+my $change_logger = get_logger("ChangeTracker");
 
 enum 'DataMode' => [qw/preset layered normal/];
 
@@ -75,22 +79,39 @@ has warp => ( is => 'ro', isa => 'Maybe[HashRef]' );
 has [qw/morph/] => ( is => 'ro', isa => 'Bool' );
 has content_warning_hash => ( is => 'rw', isa => 'HashRef',  default => sub { {}; } );
 has content_warning_list => ( is => 'rw', isa => 'ArrayRef', default => sub { []; } );
-has [qw/config_class_name cargo_class max_index index_class index_type/] =>
+has [qw/cargo_class max_index index_class index_type/] =>
     ( is => 'rw', isa => 'Maybe[Str]' );
 
+has config_model => (
+    is       => 'ro',
+    isa      => 'Config::Model',
+    weak_ref => 1,
+    lazy     => 1,
+    builder  => '_config_model'
+);
+
+sub _config_model {
+    my $self = shift;
+    my $p    = $self->instance->config_model;
+}
+
+sub config_class_name {
+    my $self = shift;
+    return $self->cargo->{config_class_name};
+}
+
 sub BUILD {
     my $self = shift;
 
     croak "Missing cargo->type parameter for element " . $self->{element_name} || 'unknown'
         unless defined $self->cargo->{type};
 
-    if ( $self->cargo->{type} eq 'node' ) {
-        $self->config_class_name( delete $self->cargo->{config_class_name} )
-            or croak "Missing cargo->config_class_name "
-            . "parameter for element "
-            . $self->element_name || 'unknown';
+    if ( $self->cargo->{type} eq 'node' and not $self->cargo->{config_class_name} ) {
+        croak "Missing cargo->config_class_name parameter for element "
+        . $self->element_name || 'unknown';
     }
-    elsif ( $self->{cargo}{type} eq 'hash' or $self->{cargo}{type} eq 'list' ) {
+
+    if ( $self->{cargo}{type} eq 'hash' or $self->{cargo}{type} eq 'list' ) {
         die "$self->{element_name}: using $self->{cargo}{type} will probably not work";
     }
 
@@ -342,6 +363,10 @@ sub notify_change {
     my $self = shift;
     my %args = @_;
 
+    $change_logger->debug( "called for ", $self->name, " from ", join( ' ', caller ),
+        " with ", join( ' ', %args ) )
+        if $change_logger->is_debug;
+
     # $idx may be undef if $self has changed, not necessarily its content
     my $idx = $args{index};
     if ( defined $idx ) {
@@ -355,7 +380,7 @@ sub notify_change {
     return if $self->instance->initial_load and not $args{really};
 
     $self->needs_check(1);
-    $self->SUPER::notify_change(@_);
+    $self->SUPER::notify_change(%args);
 }
 
 sub check {
@@ -789,11 +814,6 @@ sub auto_vivify {
         $el_class = $class;
     }
 
-    if ( not defined *{ $el_class . '::new' } ) {
-        my $file = $el_class . '.pm';
-        $file =~ s!::!/!g;
-        require $file;
-    }
 
     my @common_args = (
         element_name => $self->{element_name},
@@ -808,15 +828,10 @@ sub auto_vivify {
 
     # check parameters passed by the user
     if ( $cargo_type eq 'node' ) {
-        Config::Model::Exception::Model->throw(
-            object  => $self,
-            message => "missing 'cargo->config_class_name' " . "parameter",
-        ) unless defined $self->{config_class_name};
-
-        $item =
-            $self->{parent}->new( @common_args, config_class_name => $self->{config_class_name} );
+        $item = $self->load_node( @common_args, config_class_name => $self->config_class_name );
     }
     else {
+        Mouse::Util::load_class($el_class);
         $item = $el_class->new(@common_args);
     }
 
@@ -919,13 +934,11 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -1473,7 +1486,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
