@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyThing;
-$Config::Model::AnyThing::VERSION = '2.061';
+$Config::Model::AnyThing::VERSION = '2.082';
 use Mouse;
 
 # FIXME: must cleanup warp mechanism to implement this
@@ -20,11 +20,17 @@ use Log::Log4perl qw(get_logger :levels);
 use 5.10.1;
 
 my $logger        = get_logger("Anything");
-my $change_logger = get_logger("Anything::Change");
+my $change_logger = get_logger("ChangeTracker");
 
 has element_name => ( is => 'ro', isa => 'Str' );
 has parent       => ( is => 'ro', isa => 'Config::Model::Node', weak_ref => 1 );
-has instance     => ( is => 'ro', isa => 'Config::Model::Instance', weak_ref => 1 );
+
+has instance     => (
+    is => 'ro',
+    isa => 'Config::Model::Instance',
+    weak_ref => 1,
+    handles => [qw/show_message/]
+);
 
 # needs_check defaults to 1 to trap undef mandatory values
 has needs_check => ( is => 'rw', isa => 'Bool', default => 1 );
@@ -67,6 +73,19 @@ sub _root {
 has location       => ( is => 'ro', isa => 'Str', builder => '_location', lazy => 1 );
 has location_short => ( is => 'ro', isa => 'Str', builder => '_location_short', lazy => 1 );
 
+has backend_support_annotation => (
+    is => 'ro',
+    isa => 'Bool',
+    builder  => '_backend_support_annotation',
+    lazy     => 1
+);
+
+sub _backend_support_annotation {
+    my $self = shift;
+    # this method is overridden in Config::Model::Node
+    return $self->parent->backend_support_annotation;
+};
+
 sub notify_change {
     my $self = shift;
     my %args = @_;
@@ -147,7 +166,7 @@ sub shorten_idx {
 
     my @idx = split /\n/, $long_index // '' ;
     my $idx = shift @idx;
-    $idx .= '[truncated...]' if @idx;
+    $idx .= '[...]' if @idx;
 
     return $idx // ''; # may be undef on freebsd with perl 5.10.1 ...
 }
@@ -418,6 +437,7 @@ COMMAND:
         }
 
         if ( defined $action and not $next_obj->isa('Config::Model::AnyId') ) {
+            return if $mode eq 'loose';
             Config::Model::Exception::Model->throw(
                 object  => $obj,
                 message => "Cannot apply command '$cmd' on non hash or non list item"
@@ -435,7 +455,8 @@ COMMAND:
 
     # check element type
     if ( defined $type ) {
-        while ( @found and $found[-1]->get_type ne $type ) {
+        my @allowed = ref $type ? @$type : ($type);
+        while ( @found and not grep {$found[-1]->get_type eq $_} @allowed ) {
             Config::Model::Exception::WrongType->throw(
                 object        => $found[-1],
                 function      => 'grab',
@@ -627,7 +648,7 @@ Config::Model::AnyThing - Base class for configuration tree item
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
@@ -670,16 +691,38 @@ Returns the root node of the configuration tree.
 Returns the node location in the configuration tree. This location
 conforms with the syntax defined by L</grab()> method.
 
+=head2 location_short()
+
+Returns the node location in the configuration tree. This location truncates long
+indexes to be readable. It cannot be used by L</grab()> method.
+
 =head2 composite_name
 
 Return the element name with its index (if any). I.e. returns C<foo:bar> or
 C<foo>.
 
+=head2 composite_name_short
+
+Return the element name with its index (if any). Too long indexes are
+truncated to be readable.
+
 =head1 Annotation
 
 Annotation is a way to store miscellaneous information associated to
-each node. (Yeah... comments) These comments will be saved outside of
-the configuration file and restored the next time the command is run.
+each node. (Yeah... comments). Reading and writing annotation makes
+sense only if they can be read from and written to the configuration
+file, hence the need for the following method:
+
+=head2 backend_support_annotation
+
+Returns 1 if at least one of the backends attached to a parent node
+support to read and write annotations (aka comments) in the
+configuration file.
+
+=head2 support_annotation
+
+Returns 1 if at least one of the backends support to read and write annotations
+(aka comments) in the configuration file.
 
 =head2 annotation( [ note1, [ note2 , ... ] ] )
 
@@ -731,11 +774,17 @@ will return undef in case of problem. (default is C<strict>)
 
 =item C<type>
 
-Either C<node>, C<leaf>, C<hash> or C<list>. Returns only an object of
+Either C<node>, C<leaf>, C<hash> or C<list> or an array ref containing these
+values. Returns only an object of
 requested type. Depending on C<strict> value, C<grab> will either
 throw an exception or return the last found object of requested type.
 (optional, default to C<undef>, which means any type of object)
 
+Examples:
+
+ $root->grep(step => 'foo:2 bar', type => 'leaf')
+ $root->grep(step => 'foo:2 bar', type => ['leaf','check_list'])
+
 =item C<autoadd>
 
 When set to 1, C<hash> or C<list> configuration element are created
@@ -814,11 +863,11 @@ Notify the instance of semantic changes. Parameters are:
 
 =item old
 
-old value.
+old value. (optional)
 
 =item new
 
-new value
+new value (optional)
 
 =item path
 
@@ -833,13 +882,9 @@ element name. Default to C<$self->element_name>
 If the changed parameter is part of a hash or an array, C<index>
 contains the key or the index to get the changed parameter.
 
-=item msg
-
-change message. When not empty, old and new values are not shown.
-
 =item note
 
-note displayed along the changed values (or message)
+information about the change. Mandatory of neither old or new value are defined.
 
 =item really
 
@@ -851,6 +896,10 @@ internal parameter.
 
 =back
 
+=head2 show_message( string )
+
+Forwarded to L<Config::Model::Instance/"show_message( string )">.
+
 =head2 model_searcher ()
 
 Returns an object dedicated to search an element in the configuration
@@ -890,7 +939,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
