@@ -1,84 +1,90 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Backend::Any ;
-{
-  $Config::Model::Backend::Any::VERSION = '2.047';
-}
-
+package Config::Model::Backend::Any;
+$Config::Model::Backend::Any::VERSION = '2.082';
 use Carp;
 use strict;
-use warnings ;
-use Config::Model::Exception ;
-use Mouse ;
-use namespace::autoclean;
+use warnings;
+use Config::Model::Exception;
+use Mouse;
 
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
 
-my $logger = get_logger("Backend") ;
+my $logger = get_logger("Backend");
 
-has 'name'       => ( is => 'ro', default => 'unknown',) ;
-has 'annotation' => ( is => 'ro', isa => 'Bool', default => 0 ) ;
-has 'node'       => ( is => 'ro', isa => 'Config::Model::Node', 
-		      weak_ref => 1, required => 1 ) ;
+has 'name' => ( is => 'ro', default => 'unknown', );
+has 'annotation' => ( is => 'ro', isa => 'Bool', default => 0 );
+has 'node' => (
+    is       => 'ro',
+    isa      => 'Config::Model::Node',
+    weak_ref => 1,
+    required => 1,
+    handles => ['show_message', 'instance'],
+);
 
 sub suffix {
-    my $self = shift ;
-    $logger->info("Internal warning: suffix called for backend $self->{name}.This method can be overloaded") ;
+    my $self = shift;
+    $logger->info(
+        "Internal warning: suffix called for backend $self->{name}.This method can be overloaded"
+    );
+    return undef;
 }
 
 sub read {
-    my $self = shift ;
-    my $err = "Internal error: read not defined in backend $self->{name}." ;
-    $logger->error($err) ;
+    my $self = shift;
+    my $err  = "Internal error: read not defined in backend $self->{name}.";
+    $logger->error($err);
     croak $err;
 }
 
 sub write {
-    my $self = shift ;
-    my $err = "Internal error: write not defined in backend $self->{name}." ;
-    $logger->error($err) ;
+    my $self = shift;
+    my $err  = "Internal error: write not defined in backend $self->{name}.";
+    $logger->error($err);
     croak $err;
 }
 
 sub read_global_comments {
-    my $self = shift ;
-    my $lines = shift ;
-    my $cc = shift ; # comment character
+    my $self  = shift;
+    my $lines = shift;
+    my $cc    = shift;    # comment character
 
-    my @global_comments ;
+    my @global_comments;
 
-    while (defined ( $_ = shift @$lines ) ) {
-        next if /^$cc$cc/ ; # remove comments added by Config::Model
-        unshift @$lines,$_;
+    while ( defined( my $l = shift @$lines ) ) {
+        next if $l =~ /^$cc$cc/;    # remove comments added by Config::Model
+        unshift @$lines, $l;
         last;
     }
-    while (defined ( $_ = shift @$lines ) ) {
-        next if /^\s*$/ ; # remove empty lines
-        unshift @$lines,$_;
+    while ( defined( my $l = shift @$lines ) ) {
+        next if $l =~ /^\s*$/;      # remove empty lines
+        unshift @$lines, $l;
         last;
     }
 
-    while (defined ( $_ = shift @$lines ) ) {
-        chomp ;
+    while ( defined( my $l = shift @$lines ) ) {
+        chomp $l;
 
-        my ($data,$comment) = split /\s*$cc\s?/ , $_, 2 ;
+        my ( $data, $comment ) = split /\s*$cc\s?/, $l, 2;
 
-        push @global_comments, $comment if defined $comment ;
+        push @global_comments, $comment if defined $comment;
 
-        if (/^\s*$/ or $data) {
+        if ( $l =~ /^\s*$/ or $data ) {
             if (@global_comments) {
                 $self->node->annotation(@global_comments);
-                $logger->debug("Setting global comment with @global_comments") ;
+                $logger->debug("Setting global comment with @global_comments");
             }
-            unshift @$lines,$_ unless /^\s*$/ ; # put back any data and comment
+            # put back any data and comment
+            unshift @$lines, $l unless $l =~ /^\s*$/;
+
             # stop global comment at first blank or non comment line
             last;
         }
@@ -86,71 +92,80 @@ sub read_global_comments {
 }
 
 sub associates_comments_with_data {
-    my $self = shift ;
-    my $lines = shift ;
-    my $cc = shift ; # comment character
+    my $self  = shift;
+    my $lines = shift;
+    my $cc    = shift;    # comment character
 
-    my @result ;
-    my @comments ;
-    foreach (@$lines) {
-        next if /^$cc$cc/ ;		  # remove comments added by Config::Model
-        chomp ;
+    my @result;
+    my @comments;
+    foreach my $l (@$lines) {
+        next if $l =~ /^$cc$cc/;    # remove comments added by Config::Model
+        chomp $l;
 
-        my ($data,$comment) = split /\s*$cc\s?/, $_, 2 ;
-        push @comments, $comment        if defined $comment ;
+        my ( $data, $comment ) = split /\s*$cc\s?/, $l, 2;
+        push @comments, $comment if defined $comment;
 
-        next unless defined $data ;
+        next unless defined $data;
         $data =~ s/^\s+//g;
         $data =~ s/\s+$//g;
 
         if ($data) {
             my $note = '';
-            $note = join("\n",@comments) if @comments ;
+            $note = join( "\n", @comments ) if @comments;
             $logger->debug("associates_comments_with_data: '$note' with '$data'");
-            push @result, [  $data , $note ] ;
-            @comments = () ;
+            push @result, [ $data, $note ];
+            @comments = ();
         }
     }
-    
-    return wantarray ? @result : \@result ;
-   
+
+    return wantarray ? @result : \@result;
+
 }
 
 sub write_global_comment {
-    my ($self,$ioh,$cc) = @_ ;
+    my ( $self, $ioh, $cc ) = @_;
 
-    my $res = "$cc$cc This file was written by cme command.\n"
-        . "$cc$cc You can run 'cme edit <application>' to modify this file.\n"
-        . "$cc$cc Run 'cme list' to get the list of applications available on your system\n"
-        . "$cc$cc You may also modify the content of this file with your favorite editor.\n\n" ;
-
-    # write global comment
-    my $global_note = $self->node->annotation ;
-    if ($global_note) {
-        map { $res .= "$cc $_\n" } split /\n/,$global_note ;
-        $res .= "\n" ;
+    # no need to mention 'cme list' if current application is found
+    my $app = $self->node->instance->application ;
+    my $extra = '' ;
+    if (not $app) {
+        $extra = "$cc$cc Run 'cme list' to get the list of applications"
+            . " available on your system\n";
+        $app = '<application>';
     }
 
-    $ioh->print ($res) if defined $ioh ;
-    return $res ;
+    my $res = "$cc$cc This file was written by cme command.\n"
+        . "$cc$cc You can run 'cme edit $app' to modify this file.\n"
+        . $extra
+        . "$cc$cc You may also modify the content of this file with your favorite editor.\n\n";
+
+    # write global comment
+    my $global_note = $self->node->annotation;
+    if ($global_note) {
+        map { $res .= "$cc $_\n" } split /\n/, $global_note;
+        $res .= "\n";
+    }
+
+    $ioh->print($res) if defined $ioh;
+    return $res;
 }
 
 sub write_data_and_comments {
-    my ($self,$ioh,$cc, @data_and_comments) = @_ ;
+    my ( $self, $ioh, $cc, @data_and_comments ) = @_;
 
-    my $res  = '' ;
+    my $res = '';
     while (@data_and_comments) {
-        my ($d,$c) = splice @data_and_comments,0,2;
+        my ( $d, $c ) = splice @data_and_comments, 0, 2;
         if ($c) {
-            map { $res .= "$cc $_\n" } split /\n/,$c ;
+            map { $res .= "$cc $_\n" } split /\n/, $c;
         }
         $res .= "$d\n" if defined $d;
     }
-    $ioh->print ($res) if defined $ioh ;
-    return $res ;
+    $ioh->print($res) if defined $ioh;
+    return $res;
 }
 
-__PACKAGE__->meta->make_immutable ;
+__PACKAGE__->meta->make_immutable;
 
 1;
 
@@ -168,13 +183,12 @@ Config::Model::Backend::Any - Virtual class for other backends
 
 =head1 VERSION
 
-version 2.047
+version 2.082
 
 =head1 SYNOPSIS
 
  package Config::Model::Backend::Foo ;
  use Mouse ;
- use Log::Log4perl qw(get_logger :levels);
 
  extends 'Config::Model::Backend::Any';
 
@@ -258,11 +272,36 @@ L<Config::Model::Node>.
 
 =head2 annotation
 
-Whether the backend supports to read and write annotation. Default is
-0. Override if your backend supports annotations
+Whether the backend supports reading and writing annotation (a.k.a
+comments). Default is 0. Override this method to return 1 if your
+backend supports annotations.
+
+=head2 suffix
+
+Suffix of the configuration file. This method returns C<undef>
+
+=head2 read
+
+Read the configuration file. This method must be overridden.
+
+=head2 write
+
+Write the configuration file. This method must be overridden.
 
 =head1 Methods
 
+=head2 node
+
+Return the node (a L<Config::Model::Node>) holding this backend.
+
+=head2 instance
+
+Return the instance (a L<Config::Model::Instance>) holding this configuration.
+
+=head2 show_message( string )
+
+Show a message to STDOUT (unless overridden). Delegated to L<Config::Model::Instance/"show_message( string )">.
+
 =head2 read_global_comments( lines , comment_char)
 
 Read the global comments (i.e. the first block of comments until the first blank or non comment line) and
@@ -314,7 +353,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
