@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::IniFile;
-$Config::Model::Backend::IniFile::VERSION = '2.073';
+$Config::Model::Backend::IniFile::VERSION = '2.082';
 use Carp;
 use Mouse;
 use 5.10.0;
@@ -129,7 +129,7 @@ sub read {
 
     my @load_args = ( data => $ini_data, check => $check );
     push @load_args, split_reg => qr/$split_reg/ if $split_reg;
-    $obj->load_data(@load_args);
+    $self->load_data(@load_args);
 
     while ( my ( $k, $v ) = each %ini_comment ) {
         my $item = $obj->grab( step => $k, mode => 'loose' ) or next;
@@ -140,6 +140,12 @@ sub read {
     return 1;
 }
 
+sub load_data {
+    my $self = shift;
+    say "calling load_data on ". ref($self);
+    $self->node->load_data(@_);
+}
+
 sub set_or_push {
     my ( $self, $ref, $name, $val ) = @_;
     my $cell = $ref->{$name};
@@ -191,14 +197,146 @@ sub write {
     $ioh->print($res);
 }
 
+sub _write_list{
+    my ($self, $args, $node, $elt)  = @_ ;
+    my $res = '';
+
+    my $join_list = $args->{join_list_value};
+    my $delimiter = $args->{comment_delimiter} || '#';
+    my $obj = $node->fetch_element($elt);
+
+    my $obj_note = $obj->annotation;
+
+    if ( $join_list ) {
+        my @v = grep { length } $obj->fetch_all_values();
+        my $v = join( $join_list, @v );
+        if ( length($v) ) {
+            $logger->debug("writing joined list elt $elt -> $v");
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+        }
+    }
+    else {
+        foreach my $item ( $obj->fetch_all('custom') ) {
+            my $note = $item->annotation;
+            my $v    = $item->fetch;
+            if ( length $v ) {
+                $logger->debug("writing list elt $elt -> $v");
+                $res .=
+                    $self->write_data_and_comments( undef, $delimiter, "$elt=$v",
+                                                    $obj_note . $note );
+            }
+            else {
+                $logger->debug("NOT writing undef or empty list elt");
+            }
+        }
+    }
+    return $res;
+}
+
+sub _write_check_list{
+    my ($self, $args, $node, $elt)  = @_ ;
+    my $res = '';
+
+    my $join_check_list = $args->{join_check_list_value};
+    my $delimiter = $args->{comment_delimiter} || '#';
+    my $obj = $node->fetch_element($elt);
+
+    my $obj_note = $obj->annotation;
+
+    if ($join_check_list ) {
+        my $v = join( $join_check_list, $obj->get_checked_list() );
+        if ( length($v) ) {
+            $logger->debug("writing check_list elt $elt -> $v");
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+        }
+    }
+    else {
+        foreach my $v ( $obj->get_checked_list() ) {
+            $logger->debug("writing joined check_list elt $elt -> $v");
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+        }
+    }
+    return $res;
+}
+
+sub _write_leaf{
+    my ($self, $args, $node, $elt)  = @_ ;
+    my $res = '';
+
+    my $write_bool_as = $args->{write_boolean_as};
+    my $delimiter = $args->{comment_delimiter} || '#';
+    my $obj = $node->fetch_element($elt);
+
+    my $obj_note = $obj->annotation;
+
+    my $v = $obj->fetch;
+    if ( $write_bool_as and defined($v) and length($v) and $obj->value_type eq 'boolean' ) {
+        $v = $write_bool_as->[$v];
+    }
+    if ( defined $v and length $v ) {
+        $logger->debug("writing leaf elt $elt -> $v");
+        $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+    }
+    else {
+        $logger->debug("NOT writing undef or empty leaf elt");
+    }
+    return $res;
+}
+
+sub _write_hash {
+    my ($self, $args, $node, $elt)  = @_ ;
+    my $res = '';
+
+    my $delimiter = $args->{comment_delimiter} || '#';
+    my $obj = $node->fetch_element($elt);
+    my $obj_note = $obj->annotation;
+
+    foreach my $key ( $obj->fetch_all_indexes ) {
+        my $hash_obj = $obj->fetch_with_id($key);
+        my $note     = $hash_obj->annotation;
+        $logger->debug("writing hash elt $elt key $key");
+        my $subres = $self->_write( %$args, object => $hash_obj );
+        if ($subres) {
+            $res .= "\n"
+                . $self->write_data_and_comments( undef, $delimiter, "[$key]",
+                                                  $obj_note . $note )
+                . $subres;
+        }
+    }
+    return $res;
+}
+
+sub _write_node {
+    my ($self, $args, $node, $elt)  = @_ ;
+    my $res = '';
+
+    my $delimiter = $args->{comment_delimiter} || '#';
+    my $obj = $node->fetch_element($elt);
+    my $obj_note = $obj->annotation;
+
+    $logger->debug("writing class $elt");
+    my $subres = $self->_write( %$args, object => $obj );
+    if ($subres) {
+
+        # some INI file may have a section mapped to a node as exception to mapped in a hash
+        my $exception_name = $self->{reverse_section_map}{ $obj->location };
+        if ( defined $exception_name ) {
+            $logger->debug("writing class $exception_name from reverse_section_map");
+        }
+        my $c_name = $exception_name || $elt;
+        $res .= "\n"
+            . $self->write_data_and_comments( undef, $delimiter, "[$c_name]", $obj_note )
+            . $subres;
+    }
+    return $res;
+}
+
 sub _write {
     my $self = shift;
     my %args = @_;
 
     my $node          = $args{object};
     my $delimiter     = $args{comment_delimiter} || '#';
-    my $join_list     = $args{join_list_value};
-    my $write_bool_as = $args{write_boolean_as};
 
     $logger->debug( "called on ", $node->name );
     my $res = '';
@@ -210,48 +348,20 @@ sub _write {
         $logger->debug("first loop on elt $elt type $type");
         next if $type =~ /node/ or $type eq 'hash';
 
-        my $obj = $node->fetch_element($elt);
-
-        my $obj_note = $obj->annotation;
-
-        if ( $node->element_type($elt) eq 'list' and $join_list ) {
-            my @v = grep { length } $obj->fetch_all_values();
-            my $v = join( $join_list, @v );
-            if ( length($v) ) {
-                $logger->debug("writing list elt $elt -> $v");
-                $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
-            }
+        if ( $type eq 'list' ) {
+            $res .= $self->_write_list (\%args, $node, $elt) ;
         }
-        elsif ( $node->element_type($elt) eq 'list' ) {
-            foreach my $item ( $obj->fetch_all('custom') ) {
-                my $note = $item->annotation;
-                my $v    = $item->fetch;
-                if ( length $v ) {
-                    $logger->debug("writing list elt $elt -> $v");
-                    $res .=
-                        $self->write_data_and_comments( undef, $delimiter, "$elt=$v",
-                        $obj_note . $note );
-                }
-                else {
-                    $logger->debug("NOT writing undef or empty list elt");
-                }
-            }
+        elsif ( $type eq 'check_list') {
+            $res .= $self->_write_check_list (\%args, $node, $elt) ;
         }
-        elsif ( $node->element_type($elt) eq 'leaf' ) {
-            my $v = $obj->fetch;
-            if ( $write_bool_as and defined($v) and length($v) and $obj->value_type eq 'boolean' ) {
-                $v = $write_bool_as->[$v];
-            }
-            if ( defined $v and length $v ) {
-                $logger->debug("writing leaf elt $elt -> $v");
-                $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
-            }
-            else {
-                $logger->debug("NOT writing undef or empty leaf elt");
-            }
+        elsif ( $type eq 'leaf' ) {
+            $res .= $self->_write_leaf (\%args, $node, $elt) ;
         }
         else {
-            $logger->error("unexpected type $type for leaf elt $elt");
+            Config::Model::Exception::Model->throw(
+                error  => "unexpected type $type for leaf elt $elt",
+                object => $node
+            );
         }
     }
 
@@ -264,34 +374,10 @@ sub _write {
         my $obj_note = $obj->annotation;
 
         if ( $type eq 'hash' ) {
-            foreach my $key ( $obj->fetch_all_indexes ) {
-                my $hash_obj = $obj->fetch_with_id($key);
-                my $note     = $hash_obj->annotation;
-                $logger->debug("writing hash elt $elt key $key");
-                my $subres = $self->_write( %args, object => $hash_obj );
-                if ($subres) {
-                    $res .= "\n"
-                        . $self->write_data_and_comments( undef, $delimiter, "[$key]",
-                        $obj_note . $note )
-                        . $subres;
-                }
-            }
+            $res .= $self->_write_hash (\%args, $node, $elt) ;
         }
         else {
-            $logger->debug("writing class $elt");
-            my $subres = $self->_write( %args, object => $obj );
-            if ($subres) {
-
-                # some INI file may have a section mapped to a node as exception to mapped in a hash
-                my $exception_name = $self->{reverse_section_map}{ $obj->location };
-                if ( defined $exception_name ) {
-                    $logger->debug("writing class $exception_name from reverse_section_map");
-                }
-                my $c_name = $exception_name || $elt;
-                $res .= "\n"
-                    . $self->write_data_and_comments( undef, $delimiter, "[$c_name]", $obj_note )
-                    . $subres;
-            }
+            $res .= $self->_write_node (\%args, $node, $elt) ;
         }
     }
 
@@ -319,13 +405,11 @@ Config::Model::Backend::IniFile - Read and write config as a INI file
 
 =head1 VERSION
 
-version 2.073
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  my $model = Config::Model->new;
  $model->create_config_class (
@@ -443,16 +527,28 @@ Idem for all values.
 
 =item split_list_value
 
-Some INI values are in fact a list of items separated by a space or a comma. 
+Some INI values are in fact a list of items separated by a space or a comma.
 This parameter specifies the regex  to use to split the value into a list. This
 applies only to C<list> elements.
 
 =item join_list_value
 
 Conversely, the list element split with C<split_list_value> needs to be written
-back with a string to join them. Specify this string (usually ' ' or ', ') 
+back with a string to join them. Specify this string (usually ' ' or ', ')
 with C<join_list_value>.
 
+=item split_check_list_value
+
+Some INI values are in fact a check list of items separated by a space or a comma.
+This parameter specifies the regex to use to split the value read from the file
+into a list of items to check. This applies only to C<check_list> elements.
+
+=item join_check_list_value
+
+Conversely, the check_list element split with C<split_list_value> needs to be written
+back with a string to join them. Specify this string (usually ' ' or ', ')
+with C<join_check_list_value>.
+
 =item write_boolean_as
 
 Array ref. Reserved for boolean value. Specify how to write a boolean value. 
@@ -587,7 +683,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
