@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,7 +9,7 @@
 #
 
 package Config::Model::Backend::Yaml;
-$Config::Model::Backend::Yaml::VERSION = '2.061';
+$Config::Model::Backend::Yaml::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
@@ -24,6 +24,17 @@ my $logger = get_logger("Backend::Yaml");
 
 sub suffix { return '.yml'; }
 
+sub single_element {
+    my $self = shift;
+
+    my @elts = $self->node->children;
+    return undef if @elts != 1;
+
+    my $obj = $self->node->fetch_element($elts[0]);
+    my $type = $obj->get_type;
+    return $type =~ /^(list|hash)$/ ? $obj : undef ;
+}
+
 sub read {
     my $self = shift;
     my %args = @_;
@@ -49,8 +60,10 @@ sub read {
         return 1;
     }
 
+    my $target = $self->single_element // $self->node ;
+
     # load perl data in tree
-    $self->{node}->load_data( data => $perl_data, check => $args{check} || 'yes' );
+    $target->load_data( data => $perl_data, check => $args{check} || 'yes' );
     return 1;
 }
 
@@ -70,7 +83,15 @@ sub write {
     croak "Undefined file handle to write"
         unless defined $args{io_handle};
 
-    my $perl_data = $self->{node}->dump_as_data( full_dump => $args{full_dump} );
+    my $target = $self->single_element // $self->node ;
+
+    my $perl_data = $target->dump_as_data( full_dump => $args{full_dump} // 0);
+
+    my $size = ref($perl_data) eq 'HASH'  ? scalar keys %$perl_data
+             : ref($perl_data) eq 'ARRAY' ? scalar @$perl_data
+             :                              $perl_data ;
+    return 2 unless $size ;
+
     my $yaml = Dump $perl_data ;
 
     $args{io_handle}->print($yaml);
@@ -94,16 +115,13 @@ Config::Model::Backend::Yaml - Read and write config as a YAML data structure
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model ;
- use Log::Log4perl qw(:easy) ;
  use Data::Dumper ;
 
- Log::Log4perl->easy_init($WARN);
-
  # define configuration tree object
  my $model = Config::Model->new ;
  $model ->create_config_class (
@@ -156,10 +174,57 @@ This module is used directly by L<Config::Model> to read or write the
 content of a configuration tree written with YAML syntax in
 C<Config::Model> configuration tree.
 
-Note that undefined values are skipped for list element. I.e. if a
+Note:
+
+=over 4
+
+=item *
+
+Undefined values are skipped for list element. I.e. if a
 list element contains C<('a',undef,'b')>, the data structure will
 contain C<'a','b'>.
 
+=item *
+
+YAML file is not created (and may be deleted) when no data is to be
+written.
+
+=back
+
+=head2 Class with only one hash element
+
+If the root node contains a single hash or list element, only the
+content of this hash will be written in YAML file.
+
+For example, if a class contains:
+
+      element => [
+        baz => {
+            type => 'hash',
+            index_type => 'string' ,
+            cargo => {
+                type => 'leaf',
+                value_type => 'string',
+            },
+        },
+
+If the configuration is loaded with:
+
+ $root->load("baz:one=un baz:two=deux")
+
+Then the written YAML file will B<not> show C<baz>:
+
+ ---
+ one: un
+ two: deux
+
+Likewise, a YAML file for a class with a single list C<baz> element
+would be written with:
+
+ ---
+ - un
+ - deux
+
 =head1 CONSTRUCTOR
 
 =head2 new ( node => $node_obj, name => 'yaml' ) ;
@@ -201,7 +266,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
