@@ -1,17 +1,18 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::BackendMgr;
-$Config::Model::BackendMgr::VERSION = '2.061';
+$Config::Model::BackendMgr::VERSION = '2.082';
 use Mouse;
 
 use Carp;
+use 5.10.1;
 
 use Config::Model::Exception;
 use Data::Dumper;
@@ -45,23 +46,40 @@ has 'backend' => (
     default => sub { {} },
     handles => { set_backend => 'set', get_backend => 'get' } );
 
-#
-# New subroutine "get_cfg_dir_path" extracted - Thu Jul 11 13:47:22 2013.
-#
-sub get_cfg_dir_path {
-    my $self = shift;
-    my %args = @_;
+# Configuration directory where to read and write files. This value
+# does not override the configuration directory specified in the model
+# data passed to read and write functions.
+has config_dir => ( is => 'ro', isa => 'Maybe[Str]', required => 0 );
 
-    my $w = $args{write} || 0;
-    my $dir = $args{os_config_dir}{$^O} || $args{config_dir};
+has support_annotation => (
+    is => 'ro',
+    isa => 'Bool',
+    default => 0,
+);
+
+sub get_tuned_config_dir {
+    my ($self, %args) = @_;
+
+    my $dir = $args{os_config_dir}{$^O} || $args{config_dir} || $self->config_dir || '';
     if ( $dir =~ /^~/ ) {
         my $home = $__test_home || File::HomeDir->my_home;
         $dir =~ s/^~/$home/;
     }
 
+    $dir .= '/' if $dir and $dir !~ m(/$);
+
+    return $dir;
+}
+
+sub get_cfg_dir_path {
+    my $self = shift;
+    my %args = @_;
+
+    my $w = $args{write} || 0;
+    my $dir = $self->get_tuned_config_dir(%args);
+
     $dir = $args{root} . $dir;
 
-    $dir .= '/' unless $dir =~ m!/$!;
     if ( not -d $dir and $w and $args{auto_create} ) {
         $logger->info("creating directory $dir");
         mkpath( $dir, 0, 0755 );
@@ -102,12 +120,14 @@ sub get_cfg_file_path {
     Config::Model::Exception::Model->throw(
         error  => "backend error: empty 'config_dir' parameter (and no config_file override)",
         object => $self->node
-    ) unless $args{config_dir};
+    ) unless $args{config_dir} or $self->config_dir;
 
     my ( $dir_ok, $dir ) = $self->get_cfg_dir_path(%args);
 
     if ( defined $args{file} ) {
-        my $res = $dir . $args{file};
+        my $file = $args{file};
+        $file =~ s/&index/$self->node->index_value/eg;
+        my $res = $dir . $file;
         $logger->trace("get_cfg_file_path: returns $res");
         return ( $dir_ok, $res );
     }
@@ -256,7 +276,6 @@ sub read_config_data {
 
     # root override is passed by the instance
     my $root_dir = $instance->read_root_dir || '';
-    $root_dir .= '/' if ( $root_dir and $root_dir !~ m(/$) );
 
     croak "readlist must be array or hash ref\n"
         unless ref $readlist;
@@ -354,8 +373,7 @@ sub try_read_backend {
     my $check                = shift;
     my $backend              = shift;
 
-    my $read_dir = $read->{os_config_dir}{$^O} || $read->{config_dir} || '';
-    $read_dir .= '/' if $read_dir and $read_dir !~ m(/$);
+    my $read_dir = $self->get_tuned_config_dir(%$read);
 
     my @read_args = (
         %$read,
@@ -415,7 +433,10 @@ sub try_read_backend {
         my $suffix;
         $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
         ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => $suffix );
-        $logger->info( "Read with $backend " . $c . "::$f on $file_path" );
+        if ($logger->is_info) {
+            my $fp = defined $file_path ? " on $file_path":'' ;
+            $logger->info( "Read with $backend " . $c . "::$f".$fp);
+        }
 
         eval {
             $res = $backend_obj->$f(
@@ -425,18 +446,26 @@ sub try_read_backend {
                 object    => $self->node,
             );
         };
+
+        # only backend based on C::M::Backend::Any can support annotations
+        if ($backend_obj->can('annotation')) {
+            $self->{support_annotation} ||= $backend_obj->annotation ;
+        }
+
     }
 
     # catch eval errors done in the if-then-else block before
-    my $e;
-    if ( $e = Exception::Class->caught('Config::Model::Exception::Syntax') ) {
+    my $e = $@;
+    if ( ref($e) and $e->isa('Config::Model::Exception::Syntax') ) {
 
-        # FIXME: this is naughty. Should file a bug to add info in rethrow
-        $e->{parsed_file} = $file_path unless $e->parsed_file;
+        $e->parsed_file( $file_path) unless $e->parsed_file;
         $e->rethrow;
     }
-    elsif ( $e = Exception::Class->caught() ) {
-        ref $e ? $e->rethrow : die $e;
+    elsif ( ref $e ) {
+        $e->rethrow ;
+    }
+    elsif ( $e ) {
+        die "Backend error: $e";
     }
 
     return ( $res, $file_path );
@@ -484,8 +513,7 @@ sub auto_write_init {
             $backend .= "_file";
         }
 
-        my $write_dir = $write->{os_config_dir}{$^O} || $write->{config_dir} || '';
-        $write_dir .= '/' if $write_dir and $write_dir !~ m(/$);
+        my $write_dir = $self->get_tuned_config_dir(%$write);
 
         $logger->debug( "auto_write_init creating write cb ($backend) for ", $self->node->name );
 
@@ -591,6 +619,11 @@ sub auto_write_init {
                 };
                 $logger->warn( "write backend $backend $c" . '::' . "$f failed: $@" ) if $@;
                 $self->close_file_to_write( $@, $fh, $file_path );
+
+                if (defined $res and $res == 2) {
+                    unlink($file_path);
+                    return 1;
+                }
                 return defined $res ? $res : $@ ? 0 : 1;
             };
         }
@@ -768,14 +801,12 @@ Config::Model::BackendMgr - Load configuration node on demand
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  # Use BackendMgr to write data in perl data file
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -861,7 +892,7 @@ Config dump string (cds) in a file. I.e. a string that describes the
 content of a configuration tree is loaded from or saved in a text
 file. See L<Config::Model::Dumper>.
 
-=item ini_file
+=item IniFile
 
 INI files (written with L<Config::Model::Backend::IniFile>. See limitations in 
 L</"Limitations depending on storage">.
@@ -885,7 +916,7 @@ information back.
 
 =head2 Built-in backend
 
-C<cds_file>, C<ini_file> and C<perl_file> backend must be specified with
+C<cds_file>, C<IniFile> and C<perl_file> backend must be specified with
 mandatory C<config_dir> parameter. For instance:
 
    read_config  => { backend    => 'cds_file' , 
@@ -918,14 +949,15 @@ verbatim to C<Config::Model::Backend::Foo> methods:
                     } 
                   ]
 
-This C<Config::Model::Backend::Foo> class is expected to provide the
+This C<Config::Model::Backend::Foo> class must inherit
+L<Config::Model::Backend::Any> and is expected to provide the
 following methods:
 
 =over
 
 =item new
 
-with parameters:
+Mandatory parameters:
 
  node => ref_to_config_model_node
 
@@ -975,6 +1007,9 @@ requested by user.
 
 =head2 Custom backend
 
+Custom backend is provided to be backward compatible but should not be used
+for new project. Using a plugin backend as described above is preferred.
+
 Custom backend must be specified with a class name that will features
 the methods used to write and read the configuration files:
 
@@ -1144,7 +1179,7 @@ C<root> directory for test.
 Read callback function will be called with these parameters:
 
   object     => $obj,         # Config::Model::Node object 
-  root       => './my_test',  # fake root directory, userd for tests
+  root       => './my_test',  # fake root directory, used for tests
   config_dir => /etc/foo',    # absolute path 
   file       => 'foo.conf',   # file name
   file_path  => './my_test/etc/foo/foo.conf' 
@@ -1160,8 +1195,8 @@ The callback must return 0 on failure and 1 on successful read.
 Write callback function will be called with these parameters:
 
   object      => $obj,         # Config::Model::Node object 
-  root        => './my_test',  # fake root directory, userd for tests
-  config_dir  => /etc/foo',    # absolute path 
+  root        => './my_test',  # fake root directory, used for tests
+  config_dir  => /etc/foo',    # absolute path
   file        => 'foo.conf',   # file name
   file_path  => './my_test/etc/foo/foo.conf' 
   io_handle   => $io           # IO::File object opened in write mode 
@@ -1173,6 +1208,8 @@ The L<IO::File> object is undef if the file cannot be written to.
 
 The callback must return 0 on failure and 1 on successful write.
 
+Configuration file will be deleted if callback returns 2.
+
 =head1 CAVEATS
 
 When both C<config_dir> and C<file> are specified, this class will
@@ -1272,6 +1309,11 @@ C<< config_file => 'foo/bar.conf' >>
 
 C<write_back> will croak if no write call-back are known for this node.
 
+=head2 support_annotation
+
+Returns 1 if at least one of the backends support to read and write annotations
+(aka comments) in the configuration file.
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -1287,7 +1329,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
