@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::CheckList;
-$Config::Model::CheckList::VERSION = '2.073';
+$Config::Model::CheckList::VERSION = '2.082';
 use Mouse;
 use 5.010;
 
@@ -22,6 +22,8 @@ use Storable qw/dclone/;
 
 extends qw/Config::Model::AnyThing/;
 
+with "Config::Model::Role::WarpMaster";
+
 my $logger = get_logger("Tree::Element::CheckList");
 
 my @introspect_params = qw/refer_to computed_refer_to/;
@@ -143,6 +145,11 @@ sub set_properties {
     Config::Model::Exception::Model->throw(
         object => $self,
         error  => "Unexpected parameters :" . join( ' ', keys %args ) ) if scalar keys %args;
+
+    if ( $self->has_warped_slaves ) {
+        my $hash = $self->get_checked_list_as_hash; # force scalar context
+        $self->trigger_warp($hash, $self->fetch);
+    }
 }
 
 sub setup_choice {
@@ -225,6 +232,23 @@ sub apply_fixes {
     # supposed to be always correct. Hence apply_fixes is empty.
 }
 
+sub notify_change {
+    my $self       = shift;
+    my %args       = @_;
+
+    return if $self->instance->initial_load and not $args{really};
+
+    $self->SUPER::notify_change( %args, value_type => $self->value_type );
+
+    # notify all warped or computed objects that depends on me
+    foreach my $s ( $self->get_warped_slaves ) {
+        $logger->debug( "calling notify_change on slave ", $s->name )
+            if $logger->is_debug;
+        $s->notify_change( note => 'checklist master triggered changed' );
+    }
+}
+
+
 # does not check the validity, but check the item of the check_list
 sub check {
     my $self  = shift;
@@ -309,6 +333,15 @@ sub store {
         Config::Model::Exception::WrongValue->throw( error => $err_str, object => $self );
     }
 
+    if (    $ok
+        and $changed
+        and $self->has_warped_slaves
+        and not( $self->instance->layered or $self->instance->preset ) ) {
+        my $h = $self->get_checked_list_as_hash;
+        my $str = $self->fetch;
+        $self->trigger_warp($h , $str);
+    }
+
     return $changed;
 }
 
@@ -605,10 +638,14 @@ sub load_data {
     elsif ( ref($data) eq 'HASH' ) {
         $self->set_checked_list_as_hash($data);
     }
+    elsif ( not ref($data) ) {
+        $self->set_checked_list($data);
+    }
     else {
         Config::Model::Exception::LoadData->throw(
             object     => $self,
-            message    => "load_data called with non array ref arg",
+            message    => "check_list load_data called with unexpected type. ".
+                          "Expected plain scalar, array or hash ref",
             wrong_data => $data,
         );
     }
@@ -701,13 +738,11 @@ Config::Model::CheckList - Handle check list element
 
 =head1 VERSION
 
-version 2.073
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -1128,7 +1163,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
