@@ -1,17 +1,18 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::DumpAsData;
-$Config::Model::DumpAsData::VERSION = '2.061';
+$Config::Model::DumpAsData::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
+use 5.10.1;
 
 use Config::Model::Exception;
 use Config::Model::ObjTreeScanner;
@@ -26,20 +27,21 @@ sub dump_as_data {
     my %args      = @_;
     my $dump_node = delete $args{node}
         || croak "dump_as_data: missing 'node' parameter";
-    my $full = delete $args{full_dump};
-    $full = 1 unless defined $full;
+    my $fetch_mode = delete $args{mode} ;
     my $skip_aw = delete $args{skip_auto_write} || '';
     my $auto_v  = delete $args{auto_vivify}     || 0;
     my $ordered_hash_as_list = delete $args{ordered_hash_as_list};
     $ordered_hash_as_list = 1 unless defined $ordered_hash_as_list;
 
+    # mode and full_dump params are both accepted
+    my $full = delete $args{full_dump} // 1;
+    $fetch_mode //= 'non_upstream_default' if $full;
+    $fetch_mode //= 'custom';
+
     my $std_cb = sub {
         my ( $scanner, $data_r, $obj, $element, $index, $value_obj ) = @_;
 
-        $$data_r =
-              $full
-            ? $value_obj->fetch('non_upstream_default')
-            : $value_obj->fetch_custom;
+        $$data_r = $value_obj->fetch(mode => $fetch_mode);
     };
 
     my $check_list_element_cb = sub {
@@ -56,14 +58,15 @@ sub dump_as_data {
         # resume exploration but pass a ref on $data_ref hash element
         # instead of data_ref
         my %h;
-        my @res = map {
+        my @res;
+        foreach my $k (@keys) {
             my $v;
-            $scanner->scan_hash( \$v, $node, $element_name, $_ );
+            $scanner->scan_hash( \$v, $node, $element_name, $k );
 
-            # create the key even if $v is undef
-            $h{$_} = $v if defined $v;
-            defined $v ? ( $_, $v ) : ();
-        } @keys;
+            # don't create the key if $v is undef
+            $h{$k} = $v if defined $v;
+            push @res , $k, $v if defined $v;
+        } ;
 
         my $ordered_hash = $node->fetch_element($element_name)->ordered;
 
@@ -71,7 +74,7 @@ sub dump_as_data {
             $$data_ref = \@res if @res;
         }
         else {
-            $h{__order} = \@keys if $ordered_hash and @keys;
+            $h{'__'.$element_name.'_order'} = \@keys if $ordered_hash and @keys;
             $$data_ref = \%h if scalar %h;
         }
     };
@@ -82,22 +85,22 @@ sub dump_as_data {
         # resume exploration but pass a ref on $data_ref hash element
         # instead of data_ref
         my @a;
-        map {
+        foreach my $i (@idx) {
             my $v;
-            $scanner->scan_hash( \$v, $node, $element_name, $_ );
+            $scanner->scan_hash( \$v, $node, $element_name, $i );
             push @a, $v if defined $v;
-        } @idx;
+        }
         $$data_ref = \@a if scalar @a;
     };
 
     my $node_content_cb = sub {
         my ( $scanner, $data_ref, $node, @element ) = @_;
         my %h;
-        map {
+        foreach my $e (@element) {
             my $v;
-            $scanner->scan_element( \$v, $node, $_ );
-            $h{$_} = $v if defined $v;
-        } @element;
+            $scanner->scan_element( \$v, $node, $e );
+            $h{$e} = $v if defined $v;
+        }
         $$data_ref = \%h if scalar %h;
     };
 
@@ -249,16 +252,13 @@ Config::Model::DumpAsData - Dump configuration content as a perl data structure
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model ;
- use Log::Log4perl qw(:easy) ;
  use Data::Dumper ;
 
- Log::Log4perl->easy_init($WARN);
-
  # define configuration tree object
  my $model = Config::Model->new ;
  $model ->create_config_class (
@@ -342,6 +342,10 @@ Reference to a L<Config::Model::Node> object. Mandatory
 Also dump default values in the data structure. Useful if the dumped
 configuration data will be used by the application. (default is yes)
 
+Note that C<mode> parameter is also accepted and overrides
+C<full_dump> parameter. See L<Config::Model::Value/fetch(...)> for
+details on C<mode>.
+
 =item skip_auto_write
 
 Skip node that have a C<perl write> capability in their model. See
@@ -361,13 +365,14 @@ while keeping the key order. But it's the less readable way.
 When this parameter is 1 (default), the ordered hash is dumped as a
 list:
 
-  [ A => 'foo', B => 'bar', C => 'baz' ]
+  my_hash => [ A => 'foo', B => 'bar', C => 'baz' ]
 
 When this parameter is set as 0, the ordered hash is dumped with a
 special key that specifies the order of keys. E.g.:
 
-  { __order => [ 'A', 'B', 'C' ] ,
-    B => 'bar', A => 'foo', C => 'baz'
+  my_hash => {
+   __my_hash_order => [ 'A', 'B', 'C' ] ,
+   B => 'bar', A => 'foo', C => 'baz'
   }
 
 =back
@@ -406,7 +411,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
