@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Dumper;
-$Config::Model::Dumper::VERSION = '2.073';
+$Config::Model::Dumper::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
@@ -105,7 +105,10 @@ sub dump_tree {
         # add annotation for obj contained in hash or list
         my $note = note_quote( $value_obj->annotation );
         $$data_r .= "\n" . $pad . $name if defined $value or $note;
-        $$data_r .= '=' . $value        if defined $value;
+        if (defined $value) {
+            $value =~ s/\\n/\\\\n/g;
+            $$data_r .= '=' . $value;
+        }
         $$data_r .= '#' . $note         if $note;
     };
 
@@ -256,13 +259,11 @@ Config::Model::Dumper - Serialize data of config tree
 
 =head1 VERSION
 
-version 2.073
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model ;
- use Log::Log4perl qw(:easy) ;
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new ;
@@ -387,7 +388,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
