@@ -1,157 +1,80 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Exception;
-$Config::Model::Exception::VERSION = '2.061';
+$Config::Model::Exception::VERSION = '2.082';
 use warnings;
 use strict;
 use Data::Dumper;
+use Mouse;
+use 5.10.1;
+use Carp;
 
-use Exception::Class (
-    'Config::Model::Exception::Syntax' => {
-        isa         => 'Config::Model::Exception::Any',
-        description => 'config error',
-        fields      => [qw/object parsed_file parsed_line/],
-        description => 'syntax error',
-    },
+@Carp::CARP_NOT=qw/Config::Model::Exception Config::Model::Exception::Any/;
 
-    'Config::Model::Exception::Any' => {
-        description => 'config error',
-        fields      => [qw/object info/],
-    },
+our $trace = 0;
 
-    'Config::Model::Exception::User' => {
-        isa         => 'Config::Model::Exception::Any',
-        description => 'user error',
-    },
 
-    'Config::Model::Exception::LoadData' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'Load data structure (perl) error',
-        fields      => [qw/wrong_data/],
-    },
+use Carp qw/longmess shortmess croak/;
 
-    'Config::Model::Exception::UnavailableElement' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'unavailable element',
-        fields      => [qw/object element info function/],
-    },
-
-    'Config::Model::Exception::ObsoleteElement' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'Obsolete element',
-        fields      => [qw/object element info/],
-    },
-
-    'Config::Model::Exception::WrongType' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'wrong element type',
-        fields      => [qw/object function got_type expected_type info/],
-    },
-
-    'Config::Model::Exception::WrongValue' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'wrong value',
-    },
-
-    'Config::Model::Exception::Load' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'Load command error',
-        fields      => [qw/object command/],
-    },
-
-    'Config::Model::Exception::UnknownElement' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'unknown element',
-        fields      => [qw/object element function where info/],
-    },
-
-    'Config::Model::Exception::AncestorClass' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'unknown ancestor class',
-    },
-
-    'Config::Model::Exception::UnknownId' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'unknown identifier',
-        fields      => [qw/object element id function where/],
-    },
-
-    'Config::Model::Exception::WarpError' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'warp error',
-    },
-
-    'Config::Model::Exception::Fatal' => {
-        isa         => 'Config::Model::Exception::Any',
-        description => 'fatal error',
-    },
-
-    'Config::Model::Exception::Model' => {
-        isa         => 'Config::Model::Exception::Fatal',
-        description => 'configuration model error',
-    },
-
-    'Config::Model::Exception::ModelDeclaration' => {
-        isa         => 'Config::Model::Exception::Fatal',
-        description => 'configuration model declaration error',
-    },
-
-    'Config::Model::Exception::ConfigFile' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'error in configuration file',
-    },
-
-    'Config::Model::Exception::ConfigFile::Missing' => {
-        isa         => 'Config::Model::Exception::ConfigFile',
-        description => 'missing configuration file',
-        fields      => [qw/tried_files/]
-    },
-
-    'Config::Model::Exception::Xml' => {
-        isa         => 'Config::Model::Exception::User',
-        description => 'error in XML data',
-    },
-
-    'Config::Model::Exception::Formula' => {
-        isa         => 'Config::Model::Exception::Model',
-        description => 'error in computation formula of the ' . 'configuration model',
-    },
-
-    'Config::Model::Exception::Internal' => {
-        isa         => 'Config::Model::Exception::Fatal',
-        description => 'internal error',
-    },
-
-    'Config::Model::Exception::XmlTree' =>
-        { description => 'error while parsing XML dump of a tree' },
+use overload
+    '""' => \&full_msg_and_trace,
+    'bool' => \&is_error;
 
+has description => (
+    is => 'ro',
+    isa => 'Str',
+    lazy_build => 1
 );
 
-Config::Model::Exception::Internal->Trace(1);
-
-package Config::Model::Exception::Syntax;
-$Config::Model::Exception::Syntax::VERSION = '2.061';
-sub full_message {
+sub _build_description {
     my $self = shift;
+    return $self->_desc;
+}
 
-    my $fn   = $self->parsed_file || '?';
-    my $line = $self->parsed_line || '?';
-    my $msg  = "File $fn line $line ";
-    $msg .= "has a " . $self->description;
-    $msg .= ":\n\t" . $self->message . "\n";
+sub _desc { 'config error' }
 
+has  object => ( is => 'rw', isa => 'Ref') ;
+has  info => (is => 'rw', isa =>'Str', default => '');
+has  message => (is => 'rw', isa =>'Str', default => '');
+has  error => (is => 'rw', isa =>'Str', default => '');
+
+# without this overload, a test like if ($@) invokes '""' overload
+sub is_error { return ref ($_[0])}
+
+
+sub Trace {
+    $trace = shift;
+}
+
+sub error_or_msg {
+    my $self = shift;
+    return $self->error  || $self->message;
+}
+
+sub throw {
+    my $class = shift;
+    my $self = $class->new(@_);
+    $self->rethrow;
+}
+
+sub rethrow {
+    my $self = shift;
+    die $self;
+}
+
+sub full_msg_and_trace {
+    my $msg = shift->full_message;
+    $msg .= "Exception thrown ".longmess if $trace;
     return $msg;
 }
 
-package Config::Model::Exception::Any;
-$Config::Model::Exception::Any::VERSION = '2.061';
 sub full_message {
     my $self = shift;
 
@@ -160,27 +83,60 @@ sub full_message {
     my $msg      = "Configuration item ";
     $msg .= "'$location' "     if $location;
     $msg .= "has a " . $self->description;
-    $msg .= ":\n\t" . $self->message . "\n";
-    $msg .= $self->info . "\n" if defined $self->info;
-
+    $msg .= ":\n\t" . ($self->error || $self->message) . "\n";
+    $msg .= $self->info . "\n" if $self->info;
     return $msg;
 }
 
-sub xpath_message {
+package Config::Model::Exception::Any;
+$Config::Model::Exception::Any::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception';
+
+package Config::Model::Exception::ModelDeclaration;
+$Config::Model::Exception::ModelDeclaration::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Fatal';
+
+sub _desc {'configuration model declaration error' }
+
+package Config::Model::Exception::User ;
+$Config::Model::Exception::User::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Any';
+sub _desc {'user error' }
+
+
+## old classes below
+package Config::Model::Exception::Syntax;
+$Config::Model::Exception::Syntax::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Any';
+
+sub _desc { 'syntax error' }
+
+has [qw/parsed_file parsed_line/] => (is => 'rw', isa => 'Str');
+
+sub full_message {
     my $self = shift;
 
-    my $location = defined $self->object ? $self->object->xpath : '';
-
-    my $msg = "Configuration item ";
-    $msg .= "'$location' " if $location;
+    my $fn   = $self->parsed_file || '?';
+    my $line = $self->parsed_line || '?';
+    my $msg  = "File $fn line $line ";
     $msg .= "has a " . $self->description;
-    $msg .= ":\n\t" . $self->message . "\n";
+    $msg .= ":\n\t" . $self->error_or_msg . "\n";
 
     return $msg;
 }
 
 package Config::Model::Exception::LoadData;
-$Config::Model::Exception::LoadData::VERSION = '2.061';
+$Config::Model::Exception::LoadData::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'Load data structure (perl) error' };
+
+has wrong_data => (is => 'rw', isa => 'Ref');
 
 sub full_message {
     my $self = shift;
@@ -191,14 +147,19 @@ sub full_message {
     $msg .= "'$location' "                             if $location;
     $msg .= "(class " . $obj->config_class_name . ") " if $obj->get_type eq 'node';
     $msg .= "has a " . $self->description;
-    $msg .= ":\n\t" . $self->message . "\n";
+    $msg .= ":\n\t" . $self->error_or_msg . "\n";
     $msg .= Data::Dumper->Dump( [ $self->wrong_data ], ['wrong data'] );
 
     return $msg;
 }
 
 package Config::Model::Exception::Model;
-$Config::Model::Exception::Model::VERSION = '2.061';
+$Config::Model::Exception::Model::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Fatal';
+
+sub _desc { 'configuration model error'}
+
 
 sub full_message {
     my $self = shift;
@@ -220,13 +181,19 @@ sub full_message {
             . "', element '$element' (level $level) ";
     }
     $msg .= "has a " . $self->description;
-    $msg .= ":\n\t" . $self->message . "\n";
+    $msg .= ":\n\t" . $self->error_or_msg . "\n";
 
     return $msg;
 }
 
 package Config::Model::Exception::Load;
-$Config::Model::Exception::Load::VERSION = '2.061';
+$Config::Model::Exception::Load::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'Load command error'}
+
+has command => (is => 'rw', isa => 'ArrayRef|Str');
 
 sub full_message {
     my $self = shift;
@@ -234,20 +201,29 @@ sub full_message {
     my $location = defined $self->object ? $self->object->name : '';
     my $msg      = $self->description;
     my $cmd      = $self->command;
+    no warnings 'uninitialized';
     my $cmd_str =
-          defined $cmd && $cmd ? "'$cmd'"
+           ref($cmd)   ? join('',@$cmd)
+        : $cmd         ? "'$cmd'"
         : defined $cmd ? '<empty>'
         :                '<undef>';
-    $msg .= " in node '$location'" if $location;
+    $msg .= " in node '$location' " if $location;
     $msg .= ':';
     $msg .= "\n\tcommand: $cmd_str";
-    $msg .= "\n\t" . $self->message . "\n";
+    $msg .= "\n\t" . $self->error_or_msg . "\n";
 
     return $msg;
 }
 
 package Config::Model::Exception::UnavailableElement;
-$Config::Model::Exception::UnavailableElement::VERSION = '2.061';
+$Config::Model::Exception::UnavailableElement::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'unavailable element'}
+
+has [qw/element function/] => (is => 'rw', isa => 'Str');
+
 
 sub full_message {
     my $self = shift;
@@ -270,8 +246,22 @@ sub full_message {
     return $msg;
 }
 
+package Config::Model::Exception::AncestorClass;
+$Config::Model::Exception::AncestorClass::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'unknown ancestor class'}
+
+
 package Config::Model::Exception::ObsoleteElement;
-$Config::Model::Exception::ObsoleteElement::VERSION = '2.061';
+$Config::Model::Exception::ObsoleteElement::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'Obsolete element' }
+
+has element => (is => 'rw', isa => 'Str');
 
 sub full_message {
     my $self = shift;
@@ -290,18 +280,24 @@ sub full_message {
 }
 
 package Config::Model::Exception::UnknownElement;
-$Config::Model::Exception::UnknownElement::VERSION = '2.061';
-
+$Config::Model::Exception::UnknownElement::VERSION = '2.082';
 use Carp;
 
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'unknown element' }
+
+has [qw/element function where/] => (is => 'rw');
+
 sub full_message {
     my $self = shift;
 
     my $obj = $self->object;
 
     confess "Exception::UnknownElement: object is ", ref($obj), ". Expected a node"
-        unless $obj->isa('Config::Model::Node')
-        || $obj->isa('Config::Model::WarpedNode');
+        unless ref($obj) && ($obj->isa('Config::Model::Node')
+        || $obj->isa('Config::Model::WarpedNode'));
 
     my $class_name = $obj->config_class_name;
 
@@ -358,8 +354,29 @@ sub full_message {
     return $msg;
 }
 
+package Config::Model::Exception::WarpError;
+$Config::Model::Exception::WarpError::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'warp error'}
+
+package Config::Model::Exception::Fatal;
+$Config::Model::Exception::Fatal::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Any';
+
+sub _desc { 'fatal error' }
+
+
 package Config::Model::Exception::UnknownId;
-$Config::Model::Exception::UnknownId::VERSION = '2.061';
+$Config::Model::Exception::UnknownId::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'unknown identifier'}
+
+has [qw/element id function where/] => (is => 'rw', isa => 'Str');
 
 sub full_message {
     my $self = shift;
@@ -386,8 +403,22 @@ sub full_message {
     return $msg;
 }
 
+package Config::Model::Exception::WrongValue;
+$Config::Model::Exception::WrongValue::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'wrong value'};
+
+
 package Config::Model::Exception::WrongType;
-$Config::Model::Exception::WrongType::VERSION = '2.061';
+$Config::Model::Exception::WrongType::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'wrong element type' };
+
+has [qw/function got_type expected_type/] => (is => 'rw', isa => 'Str');
 
 sub full_message {
     my $self = shift;
@@ -411,31 +442,43 @@ sub full_message {
     return $msg;
 }
 
+package Config::Model::Exception::ConfigFile;
+$Config::Model::Exception::ConfigFile::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::User';
+
+sub _desc { 'error in configuration file' }
+
 package Config::Model::Exception::ConfigFile::Missing;
-$Config::Model::Exception::ConfigFile::Missing::VERSION = '2.061';
+$Config::Model::Exception::ConfigFile::Missing::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::ConfigFile';
+
+sub _desc { 'missing configuration file'}
+
+has tried_files => (is => 'rw', isa => 'ArrayRef');
 
 sub full_message {
     my $self = shift;
 
-    my $msg = "Error: cannot find configuration file " . join ' or ', @{ $self->tried_files };
+    my $msg = "Error: cannot find configuration file " . join (' or ', @{ $self->tried_files });
 
     return $msg . "\n";
 }
 
-package Config::Model::Exception::Xml;
-$Config::Model::Exception::Xml::VERSION = '2.061';
+package Config::Model::Exception::Formula;
+$Config::Model::Exception::Formula::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Model';
 
-sub full_message {
-    my $self = shift;
+sub _desc { 'error in computation formula of the configuration model'}
 
-    my $obj = $self->object;
-    my $msg = $self->message;
+package Config::Model::Exception::Internal;
+$Config::Model::Exception::Internal::VERSION = '2.082';
+use Mouse;
+extends 'Config::Model::Exception::Fatal';
 
-    $msg .= "\n\t" . join( "\n\t", map ( $_->xpath_message, $self->object->errors ) )
-        if defined $self->object;
-
-    return $msg . "\n";
-}
+sub _desc { 'internal error' }
 
 1;
 
@@ -453,25 +496,30 @@ Config::Model::Exception - Exception mechanism for configuration model
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
- # internal
+  use  Config::Model::Exception;
+
+  # later
+  my $kaboom = 1;
+  Config::Model::Exception::Model->throw(
+      error  => "Went kaboom",
+      object => $self
+  ) if $kaboom;
 
 =head1 DESCRIPTION
 
-You must read L<Exception::Class> before reading on.
+This module creates exception classes used by L<Config::Model>.
 
-This module creates all the exception class used by L<Config::Model>.
-
-All exception class name begins with C<Config::Model::Exception::>
+All exception class name begins with C<Config::Model::Exception>
 
 The exception classes are:
 
 =over
 
-=item C<Any>
+=item C<Config::Model::Exception>
 
 Base class. It accepts an C<object> argument. The user must pass the
 reference of the object where the exception occurred. The object name
@@ -484,17 +532,10 @@ will be used to generate the error message.
 =head1 How to get trace
 
 By default, most of the exceptions will not print out the stack
-trace. For debug purpose, you can force a stack trace.
+trace. For debug purpose, you can force a stack trace for all
+exception classes:
 
-For instance, if you want a stack trace for an "unknown element"
-error, you must add this line in your script:
-
-  Config::Model::Exception::UnknownElement->Trace(1) ;
-
-If you're not sure which class to trace, add this line in your
-script:
-
-  Config::Model::Exception::Any->Trace(1) ;
+  Config::Model::Exception->Trace(1) ;
 
 =head1 AUTHOR
 
@@ -506,7 +547,6 @@ L<Config::Model>,
 L<Config::Model::Instance>, 
 L<Config::Model::Node>,
 L<Config::Model::Value>
-L<Exception::Class>
 
 =head1 AUTHOR
 
@@ -514,7 +554,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
