@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::FuseUI;
-$Config::Model::FuseUI::VERSION = '2.073';
+$Config::Model::FuseUI::VERSION = '2.082';
 # there's no Singleton with Mouse
 use Mouse;
 
@@ -217,7 +217,8 @@ sub write {
     substr $v, $off, length($buf), $buf;
     chomp $v unless ( $type eq 'leaf' and $obj->value_type eq 'string' );
     $logger->debug("FuseUI write stores '$v'");
-    $obj->store( value => $v );
+    $obj->store( value => $v, check => 'skip', say_dont_warn => 1 );
+
     return length($buf);
 }
 
@@ -326,28 +327,26 @@ Config::Model::FuseUI - Fuse virtual file interface for Config::Model
 
 =head1 VERSION
 
-version 2.073
+version 2.082
 
 =head1 SYNOPSIS
 
  # command line
- mkdir fuse_dir
- config-edit -application popcon -ui fuse -fuse_dir fusedir 
- ll fuse_dir
- fusermount -u fuse_dir
- 
+ mkdir mydir
+ cme fusefs popcon -fuse-dir mydir
+ ll mydir
+ fusermount -u mydir
+
  # programmatic
  use Config::Model ;
  use Config::Model::FuseUI ;
- use Log::Log4perl qw(:easy) ; 
- 
- Log::Log4perl->easy_init($WARN); 
+
  my $model = Config::Model -> new; 
  my $root = $model -> instance (root_class_name => "PopCon") -> config_root ; 
- my $ui = Config::Model::FuseUI->new( root => $root, mountpoint => "fuse_dir" ); 
+ my $ui = Config::Model::FuseUI->new( root => $root, mountpoint => "mydir" ); 
  $ui -> run_loop ;  # blocking call
- 
- # explore fuse_dir in another terminal then umount fuse_dir directory
+
+ # explore mydir in another terminal then umount mydir directory
 
 =head1 DESCRIPTION
 
@@ -356,7 +355,7 @@ parameter of your configuration file is mapped to a file.
 
 =head1 Example 
 
- $ perl -Ilib config-edit -ui fuse -fuse_dir fused -appli popcon 
+ $ cme fusefs popcon -fuse-dir fused
  Mounting config on fused in background.
  Use command 'fusermount -u fused' to unmount
  $ ll fused
@@ -373,8 +372,19 @@ parameter of your configuration file is mapped to a file.
 
 =head1 BUGS
 
+=over
+
+=item *
+
 For some configuration, mapping each parameter to a file may lead to a high number of files.
 
+=item *
+
+The content of a file is when writing a wrong value. I.e. the files is
+empty and the old value is lost.
+
+=back
+
 =head1 constructor
 
 =head1 new (...)
@@ -410,7 +420,7 @@ unmounted (with C<fusermount -u mount_point> command)
 
 =head1 SEE ALSO
 
-L<Fuse>, L<Config::Model>
+L<Fuse>, L<Config::Model>, L<cme>
 
 =head1 AUTHOR
 
@@ -418,7 +428,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
