@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::IdElementReference;
-$Config::Model::IdElementReference::VERSION = '2.061';
+$Config::Model::IdElementReference::VERSION = '2.082';
 use Mouse;
 
 use Carp;
@@ -86,14 +86,12 @@ sub get_choice_from_refered_to {
 
         my $refered_to = eval { $config_elt->grab("@path"); };
 
-        if ($@) {
+        if (ref $@) {
             my $e = $@;
-
             # don't use $e->full_description as it will recurse badly
-            my $msg = $e ? $e->description : '';
             Config::Model::Exception::Model->throw(
                 object => $config_elt,
-                error  => "'refer_to' parameter with path '@path': " . $msg
+                error  => "'refer_to' parameter with path '@path': " .$e->description
             );
         }
 
@@ -192,15 +190,13 @@ Config::Model::IdElementReference - Refer to id element(s) and extract keys
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  # synopsis shows an example of model of a network to use references
- 
+
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  my $model = Config::Model->new;
 
@@ -382,7 +378,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
