@@ -1,20 +1,21 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Instance;
-$Config::Model::Instance::VERSION = '2.061';
+$Config::Model::Instance::VERSION = '2.082';
 #use Scalar::Util qw(weaken) ;
 
 use 5.10.1;
 use Mouse;
 use Mouse::Util::TypeConstraints;
 use MouseX::StrictConstructor;
+with "Config::Model::Role::NodeLoader";
 
 use Text::Diff;
 use File::Path;
@@ -38,6 +39,8 @@ my $change_logger = get_logger("Anything::Change");
 
 has [qw/root_class_name/] => ( is => 'ro', isa => 'Str', required => 1 );
 
+sub location { return "in instance" }
+
 has config_model => (
     is       => 'ro',
     isa      => 'Config::Model',
@@ -158,7 +161,19 @@ sub has_warning {
 
 has on_change_cb => (
     is  => 'rw',
-    isa => 'Maybe[CodeRef]',
+    traits    => ['Code'],
+    isa       => 'CodeRef',
+    default   => sub { sub { } },
+);
+
+has on_message_cb => (
+    traits    => ['Code'],
+    is        => 'rw',
+    isa       => 'CodeRef',
+    default   => sub { sub { say @_; } },
+    handles   => {
+        show_message => 'execute',
+    },
 );
 
 # initial_load mode: when data is loaded the first time
@@ -197,23 +212,30 @@ has [qw/name application root_dir backend backup/] => (
     is  => 'ro',
     isa => 'Maybe[Str]',
 );
+
+has read_root_dir => (
+    is  => 'ro',
+    isa => 'Str',
+    lazy_build => 1,
+);
+
+sub _build_read_root_dir {
+    my $self = shift;
+    my $root_dir = $self->root_dir // '';
+    # cleanup paths
+    $root_dir .= '/' if $root_dir and $root_dir !~ m!/$! ;
+    return $root_dir;
+}
+
 # config_file cannot be a Path::Tiny object: it may be a file name
 # relative to a directory only known by a backend (e.g. a patch in
 # debian/patches directory)
 has config_file => (is  => 'ro', isa => 'Maybe[Str]');
 
+has config_dir => (is  => 'ro', isa => 'Maybe[Str]');
+
 has skip_read => ( is => 'ro', isa => 'Bool', default => 0 );
 
-sub BUILD {
-    my $self = shift;
-
-    #carp "instance new: force_load is deprecated" if defined $args{force_load} ;
-    #$read_check = 'no' if delete $args{force_load} ;
-
-    # cleanup paths
-    map { $self->{$_} .= '/' if defined $self->{$_} and $self->{$_} !~ m!/$! } qw/root_dir/;
-}
-
 has tree => (
     is      => 'ro',
     isa     => 'Config::Model::Node',
@@ -225,7 +247,7 @@ has tree => (
 sub reset_config {
     my $self = shift;
 
-    return Config::Model::Node->new(
+    return $self->load_node (
         config_class_name => $self->{root_class_name},
         instance          => $self,
         container         => $self,
@@ -352,22 +374,18 @@ sub iterator {
 
 sub read_directory {
     carp "read_directory is deprecated";
-    return shift->{root_dir};
-}
-
-sub read_root_dir {
-    return shift->{root_dir};
+    return shift->read_root_dir;
 }
 
 sub write_directory {
     my $self = shift;
     carp "write_directory is deprecated";
-    return $self->{root_dir};
+    return $self->read_root_dir;
 }
 
 sub write_root_dir {
     my $self = shift;
-    return $self->{root_dir};
+    return $self->read_root_dir;
 }
 
 # FIXME: record changes to implement undo/redo ?
@@ -377,9 +395,17 @@ sub notify_change {
     if ( $change_logger->is_debug ) {
         $change_logger->debug( "in instance ", $self->name, ' for path ', $args{path} );
     }
+
+    foreach my $obsolete (qw/note_only msg/) {
+        if ( my $m = delete $args{$obsolete} ) {
+            carp "notify_change: param $obsolete is obsolete ($m)";
+            $args{note} //='';
+            $args{note} .= $m;
+        }
+    }
+
     $self->add_change( \%args );
-    my $cb = $self->on_change_cb;
-    $cb->(@_) if $cb;
+    $self->on_change_cb->( %args );
 }
 
 sub list_changes {
@@ -388,29 +414,28 @@ sub list_changes {
     my @all;
 
     foreach my $c (@$l) {
-        my $path = $c->{path};
-
-        if ( my $m = delete $c->{msg} ) {
-            push @all, "$path : $m" . ( $c->{note} ? " # $c->{note}" : '' );
-            next;
-        }
-
-        # don't list change without further info (like nodes)
-        next unless keys %$c > 1;
+        my $path = $c->{path} ;
+        $path .= ': ' if $path;
 
         my $vt = $c->{value_type} || '';
         my ( $o, $n ) = map { $_ // '<undef>'; } ( $c->{old}, $c->{new} );
 
         if ( $vt eq 'string' and ( $o =~ /\n/ or $n =~ /\n/ ) ) {
-
             # append \n if needed so diff works as expected
             map { $_ .= "\n" unless /\n$/; } ( $o, $n );
             my $diff = diff \$o, \$n;
-            push @all, "$path :" . ( $c->{note} ? " # $c->{note}" : '' ) . "\n" . $diff;
+            push @all, $path . ( $c->{note} ? " # $c->{note}" : '' ) . "\n" . $diff;
         }
         elsif ( defined $c->{old} or defined $c->{new} ) {
             map { s/\n.*/.../s; } ( $o, $n );
-            push @all, "$path: '$o' -> '$n'" . ( $c->{note} ? " # $c->{note}" : '' );
+            push @all, $path."'$o' -> '$n'" . ( $c->{note} ? " # $c->{note}" : '' );
+        }
+        elsif (defined $c->{note}){
+            push @all, $path.$c->{note};
+        }
+        else {
+            # something's unexpected with the call to notify_change
+            push @all, "changed ".join(' ', each %$c);
         }
     }
 
@@ -421,7 +446,7 @@ sub say_changes {
     my $self    = shift;
     my @changes = $self->list_changes;
     say "\n",
-        join( "\n- ", "Changes applied to " . $self->application . " configuration:", @changes ),
+        join( "\n- ", "Changes applied to " . ($self->application // $self->name) . " configuration:", @changes ),
         "\n"
         if @changes;
     return @changes;
@@ -464,6 +489,29 @@ sub write_back {
     $self->clear_changes;
 }
 
+sub update {
+    my ($self, %args) = @_;
+
+    my @msgs ;
+    my $hook = sub {
+        my ($scanner, $data_ref,$node,@element_list) = @_;
+        if ($node->can('update')) {
+            say "Calling update on ",$node->name, ' ',$node->config_class_name, " $node"
+                unless $args{quiet};
+            push (@msgs, $node->update(%args))
+        } ;
+    };
+
+    my $root = $self->config_root ;
+
+    Config::Model::ObjTreeScanner->new(
+        node_content_hook => $hook,
+        leaf_cb => sub { }
+    )->scan_node( \@msgs, $root );
+
+    return @msgs;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -482,14 +530,12 @@ Config::Model::Instance - Instance of configuration tree
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
  use File::Path ;
- Log::Log4perl->easy_init($WARN);
 
  # setup a dummy popcon conf file
  my $wr_dir = '/tmp/etc/';
@@ -537,7 +583,14 @@ change the "root" directory with C<root_dir> parameter:
 
 =item root_dir
 
-Pseudo root directory where to read I<and> write configuration files
+Pseudo root directory where to read I<and> write configuration
+files. Configuration directory specified in model or with
+C<config_dir> option is appended to this root directory
+
+=item config_dir
+
+Directory to read or write configuration file. This parameter must be
+supplied if not provided by the configuration model.
 
 =item backend
 
@@ -557,6 +610,11 @@ configuration tree.
 Call back this function whenever C<notify_change> is called. Called with
 arguments: C<< name => <root node element name>, index => <index_value> >>
 
+=item on_message_cb
+
+Call back this function when L<show_message> is called. By default,
+messages will be displayed on STDOUT.
+
 =item error_paths
 
 Returns a list of tree items that currently have an error.
@@ -588,6 +646,11 @@ Returns the root object of the configuration tree.
 
 Returns how to check read files.
 
+=head2 show_message( string )
+
+Display the message on STDOUT unless a custom function was passed to
+C<on_message_cb> parameter.
+
 =head2 reset_config
 
 Destroy current configuration tree (with data) and returns a new tree with
@@ -708,7 +771,8 @@ C<write_back> method.
 
 =head2 notify_change
 
-Notify that some data has changed in the tree. 
+Notify that some data has changed in the tree. See
+L<Config::Model::AnyThing/notify_change(...)> for more details.
 
 =head2 write_back ( ... )
 
@@ -735,7 +799,8 @@ See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>.
 
 =head2 needs_save
 
-Returns 1 (or more) if the instance contains data that needs to be saved.
+Returns 1 (or more) if the instance contains data that needs to be
+saved. I.e some change were done in the tree that needs to be saved.
 
 =head2 list_changes
 
@@ -750,6 +815,12 @@ Print all changes on STDOUT and return the list of changes.
 
 Returns the number of warning found in the elements of this configuration instance.
 
+=head2 update( quiet => (0|1), %args )
+
+Try to run update command on all nodes of the configuration tree. Node
+without C<update> method are ignored. C<update> will C<say> a message
+otherwise (unless C<quiet> is true).
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -768,7 +839,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
