@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Iterator;
-$Config::Model::Iterator::VERSION = '2.073';
+$Config::Model::Iterator::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
@@ -227,8 +227,8 @@ sub leaf_cb {
     my $result;
     eval { $result = $value_obj->fetch(); };
 
-    my $e;
-    if ( $e = Exception::Class->caught('Config::Model::Exception::User') ) {
+    my $e =  $@;
+    if ( ref $e and $e->isa('Config::Model::Exception::User') ) {
 
         # ignore errors that has just been catched and call user call-back
         $logger->info(
@@ -239,12 +239,13 @@ sub leaf_cb {
         );
         $user_leaf_cb->( $self, $data_r, $node, $element, $index, $value_obj, $e->error );
     }
-    elsif ( $e = Exception::Class->caught() ) {
+    elsif ( ref $e ) {
         $e->rethrow;
-
         # does not return ...
     }
-
+    elsif ($e) {
+        die "Iterator failed on value object: $e";
+    }
 }
 
 sub go_forward {
@@ -275,13 +276,11 @@ Config::Model::Iterator - Iterates forward or backward a configuration tree
 
 =head1 VERSION
 
-version 2.073
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -460,7 +459,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
