@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ListId;
-$Config::Model::ListId::VERSION = '2.061';
+$Config::Model::ListId::VERSION = '2.082';
 use 5.10.1;
 use Mouse;
 
@@ -197,8 +197,10 @@ sub store_set {
     }
 
     # and delete unused items
-    my $max = scalar @{ $self->{data} };
-    splice @{ $self->{data} }, $idx, $max - $idx;
+    my $ref = $self->{data};
+    while (scalar @$ref > $idx) {
+        $self->delete($#$ref);
+    }
 }
 
 # store without any check
@@ -413,7 +415,7 @@ sub remove {
     ) unless $idx =~ /^\d+$/;
 
     $self->delete_data_mode( index => $idx );
-    $self->notify_change;
+    $self->notify_change(note => "removed idx $idx");
     splice @{ $self->{data} }, $idx, 1;
 }
 
@@ -497,7 +499,7 @@ Config::Model::ListId - Handle list element for configuration model
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
@@ -659,7 +661,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
