@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Loader;
-$Config::Model::Loader::VERSION = '2.061';
+$Config::Model::Loader::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
@@ -20,7 +20,7 @@ use Log::Log4perl qw(get_logger :levels);
 my $logger = get_logger("Loader");
 
 ## load stuff, similar to grab, but used to set items in the tree
-## staring from this node
+## starting from this node
 
 sub new {
     bless {}, shift;
@@ -88,9 +88,15 @@ sub load {
     } while ( $ret eq 'root' );
 
     if (@command) {
-        my $str = "Error: command '@command' was not executed, you may have"
-            . " specified too many '-' in your command (ret is $ret)\n";
+        my $str = "Error: could not execute the required command, ";
+        if ($command[0] =~ m!^/([\w-]+)!) {
+            $str .=  "the searched item '$1' was not found" ;
+        }
+        else {
+            $str .= "you may have specified too many '-' in your command";
+        }
         Config::Model::Exception::Load->throw(
+            command => $command[0],
             error  => $str,
             object => $node
         ) if $check eq 'yes';
@@ -117,7 +123,7 @@ sub _split_cmd {
 	 (?:
             (:~|:-[=~]?|:=~|:\.\w+|:[=<>@]?|~)       # action
             (?:
-                  (?: \( ([^)]+)  \) )  # capture parameters between braces
+                  (?: \( ( $quoted_string | [^)]+ ) \) )  # capture parameters between braces
                 | (
                     /[^/]+/      # regexp
                     | (?:
@@ -191,6 +197,18 @@ sub _load {
             return 'up';
         }
 
+        if ( $cmd =~ m!^/([\w-]+)! ) {
+            my $search = $1;
+            if ($node->has_element($search)) {
+                $logger->debug("_load: search found node with element $search");
+                $cmd =~ s!^/!! ;
+            } else {
+                $logger->debug("_load: searching node with element $search, going up");
+                unshift @$cmdref, $cmd;
+                return 'up';
+            }
+        }
+
         my @instructions = _split_cmd($cmd);
         my ( $element_name, $action, $function_param, $id, $subaction, $value, $note ) =
             @instructions;
@@ -266,6 +284,7 @@ sub _load {
 
         $logger->debug("_load: calling $element_type loader on element $element_name");
         my $ret = $self->$method( $node, $check, \@instructions, $cmdref );
+        $logger->debug("_load: $element_type loader on element $element_name returned $ret");
         die "Internal error: method dispatched for $element_type returned an undefined value "
             unless defined $ret;
 
@@ -328,7 +347,13 @@ sub _walk_node {
 }
 
 sub unquote {
-    map { s/^"// && s/"$// && s!\\"!"!g if defined $_; } @_;
+    map {
+        if (defined $_) {
+            s/(?<!\\)\\n/\n/g;
+            s/\\\\/\\/g;
+            s/^"// && s/"$// && s!\\"!"!g;
+        }
+    } @_;
 }
 
 sub _load_check_list {
@@ -370,6 +395,8 @@ sub _load_check_list {
 
 }
 
+# sub is called with  ( $self, $element, $check, $instance, @function_args )
+# function_args are the arguments passed to the load command
 my %dispatch_action = (
     list_leaf => {
         ':.sort'          => sub { $_[1]->sort; },
@@ -382,6 +409,7 @@ my %dispatch_action = (
     'hash_*' => {
         ':.sort'          => sub { $_[1]->sort; },
         ':@'              => sub { $_[1]->sort; },
+        ':.copy'          => sub { $_[1]->copy( $_[4], $_[5] ); },
     },
     leaf => {
         ':-=' => \&_remove_by_value,
@@ -572,11 +600,15 @@ sub _load_hash {
     if ( $action eq ':~' ) {
         my @keys = $element->fetch_all_indexes;
         my $ret  = 'ok';
-        $logger->debug("_load_hash: looping with regex $id on keys @keys");
-        $id =~ s!^/!!;
-        $id =~ s!/$!!;
+        $id =~ s!^/|/$!!g if $id;
+        my @loop_on = $id ? grep { /$id/ } @keys : @keys;
+        if ($logger->is_debug) {
+            my $str = $id ? " with regex /$id/" : '';
+            $logger->debug("_load_hash: looping$str on keys @loop_on");
+        }
+
         my @saved_cmd = @$cmdref;
-        foreach my $loop_id ( grep /$id/, @keys ) {
+        foreach my $loop_id ( @loop_on ) {
             @$cmdref = @saved_cmd;    # restore command before loop
             $logger->debug("_load_hash: loop on id $loop_id");
             my $sub_elt = $element->fetch_with_id($loop_id);
@@ -597,11 +629,14 @@ sub _load_hash {
                 );
             }
 
-            if ( $ret eq 'error' or $ret eq 'root' ) { return $ret; }
+            $logger->debug("_load_hash: loop on id $loop_id returned $ret (left cmd: @$cmdref)");
+            if ( $ret eq 'error') { return $ret; }
         }
         return $ret;
     }
 
+    my @f_args = grep { defined } ( ( $f_arg // $id // '' ) =~ /([^,"]+)|"([^"]+)"/g );
+
     if ( defined $action ) {
         my $dispatch =
                $dispatch_action{ 'hash_' . $cargo_type }{$action}
@@ -609,7 +644,8 @@ sub _load_hash {
             || $dispatch_action{$cargo_type}{$action}
             || $dispatch_action{'fallback'}{$action};
         if ($dispatch) {
-            $dispatch->( $self, $element, $check, $inst, $id );
+            # todo missing arguments
+            $dispatch->( $self, $element, $check, $inst, @f_args );
             return 'ok';
         }
     }
@@ -659,9 +695,28 @@ sub _load_leaf {
     my $element = $node->fetch_element( name => $element_name, check => $check );
     $self->_load_note( $element, $note, $inst, $cmdref );
 
-    if ( defined $action and $action eq '~' and $element->isa('Config::Model::Value') ) {
-        $logger->debug("_load_leaf: action '$action' deleting value");
-        $element->store(undef);
+    if ( defined $action and $element->isa('Config::Model::Value')) {
+        if ($action eq '~') {
+            $logger->debug("_load_leaf: action '$action' deleting value");
+            $element->store(value => undef, check => $check);
+        }
+        elsif ($action eq ':') {
+            Config::Model::Exception::Load->throw(
+                object  => $element,
+                command => $inst,
+                error   => "Error: list or hash command (':') detected on a leaf."
+                . "(element '" . $element->name . "')"
+            );
+        }
+        else {
+            Config::Model::Exception::Load->throw(
+                object  => $element,
+                command => $inst,
+                error   => "Load error on leaf with "
+                . "'$element_name$action$id' command "
+                . "(element '" . $element->name . "')"
+            );
+        }
     }
 
     return 'ok' unless defined $subaction;
@@ -715,6 +770,7 @@ sub _load_value {
         return undef;
     }
 
+    $logger->debug("_load_value: done returns ok");
     return 'ok';
 }
 
@@ -734,13 +790,11 @@ Config::Model::Loader - Load serialized data into config tree
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -836,21 +890,22 @@ separated by spaces. These actions can be divided in 4 groups:
 
 =over
 
-=item navigation
+=item *
 
-Moving up and down the configuration tree.
+navigation: moving up and down the configuration tree.
 
-=item list and hash operation
+=item *
 
-select, add or delete hash or list item (also known as C<id> items)
+list and hash operation: select, add or delete hash or list item (also
+known as C<id> items)
 
-=item leaf operation
+=item *
 
-select, modify or delecte leaf value
+leaf operation: select, modify or delecte leaf value
 
-=item annotation
+=item *
 
-modify or delete configuration annotation (aka comment)
+annotation: modify or delete configuration annotation (aka comment)
 
 =back
 
@@ -870,6 +925,11 @@ Go to the root node of the configuration tree.
 
 Go down using C<xxx> element. (For C<node> type element)
 
+=item /xxx
+
+Go up until the element C<xxx> is found. This search can be combined with one of the
+command specified below, e.g C</a_string="foo bar">
+
 =back
 
 =head2 list and hash operation
@@ -879,19 +939,37 @@ Go down using C<xxx> element. (For C<node> type element)
 =item xxx:yy
 
 Go down using C<xxx> element and id C<yy> (For C<hash> or C<list>
-element with C<node> cargo_type)
+element with C<node> cargo_type). Literal C<\n> will be replaced by
+real C<\n> (LF in Unix).
 
-=item xxx:~/yy/
+=item xxx:~yy
 
-Go down using C<xxx> element and loop over the ids that match the regex.
-(For C<hash>)
+Go down using C<xxx> element and loop over the ids that match the regex
+specified by C<yy>. (For C<hash>).
 
 For instance, with C<OpenSsh> model, you could do
 
- Host:~/.*.debian.org/ user='foo-guest'
+ Host:~"/.*.debian.org/" user='foo-guest'
 
 to set "foo-user" users for all your debian accounts.
 
+The leading and trailing '/' may be omitted. Be sure to surround the
+regexp with double quote if space are embedded in the regex.
+
+Note that the loop ends when the load command goes above the element
+where the loop is executed. For instance, the instruction below will
+try to execute C<DX=BV> and C<int_v=9> for all elements of C<std_id> hash:
+
+ std_id:~/^\w+$/ DX=Bv int_v=9
+
+In the examples below only C<DX=BV> is executed by the loop:
+
+ std_id:~/^\w+$/ DX=Bv - int_v=9
+ std_id:~/^\w+$/ DX=Bv ! int_v=9
+
+The loop is done on all elements of the hash when no value is passed
+after "C<:~>" (mnemonic: an empty regexp matches any value).
+
 =item xxx:-yy
 
 Delete item referenced by C<xxx> element and id C<yy>. For a list,
@@ -911,7 +989,7 @@ Will not complain if no value were deleted.
 
 =item xxx:=~s/yy/zz/
 
-Substitute a value with another
+Substitute a value with another. Perl switches can be used(e.g. C<xxx:=~s/yy/zz/gi>)
 
 =item xxx:<yy or xxx:.push(yy)
 
@@ -955,6 +1033,10 @@ identified by key C<yy> to value C<zz>.
 
 Using C<xxx:~/yy/=zz> is also possible.
 
+=item xxx:.copy(yy,zz)
+
+copy item C<yy> in C<zz>
+
 =back
 
 =head2 leaf operation
@@ -964,11 +1046,10 @@ Using C<xxx:~/yy/=zz> is also possible.
 =item xxx=zz
 
 Set element C<xxx> to value C<yy>. load also accepts to set elements
-with a quoted string. (For C<leaf> element)
+with a quoted string. (For C<leaf> element) Literal C<\n> will be replaced by
+real C<\n> (LF in Unix). Literal C<\\> will be replaced by C<\>.
 
-For instance C<foo="a quoted string">. Note that you cannot embed
-double quote in this string. I.e C<foo="a \"quoted\" string"> will
-fail.
+For instance C<foo="a quoted string"> or C<foo="\"bar\" and \"baz\"">.
 
 =item xxx=~s/foo/bar/
 
@@ -1011,7 +1092,39 @@ C<foo#comment:bar> is B<not> valid.
 
 You can surround indexes and values with double quotes. E.g.:
 
-  a_string="\"titi\" and \"toto\""
+  a_string="\"foo\" and \"bar\""
+
+=head1 Examples
+
+You can use L<cme> to modify configuration with C<cme modify> command.
+
+For instance, if L<Config::Model::Ssh> is installed, you can run:
+
+ cme modify ssh 'ControlMaster=auto ControlPath="~/.ssh/master-%r@%n:%p"'
+
+To delete C<Host *> entry:
+
+ cme modify ssh 'Host:-"*"'
+
+To specify 2 C<Host> with a single command:
+
+ cme modify ssh 'Host:"foo* bar*" ForwardX11=yes HostName="foo.com" - Host:baz HostName="baz.com"'
+
+Note the 'C<->' used to go up one node before "C<Host:baz>". In this
+case, "up one node" leads to the "root node", so "C<!>" could also be
+used instead of "C<->":
+
+ cme modify ssh 'Host:"foo* bar*" ForwardX11=yes HostName="foo.com" ! Host:baz HostName="baz.com"'
+
+Let's modify now the host name of using a C<.org> domain instead of
+C<.com>. The C<:~> operator uses a regexp to loop over several Host
+entries:
+
+ cme modify ssh 'Host:~/ba[rz]/ HostName=~s/.com$/.org/'
+
+Now that ssh config is mucked up with dummy entries, let's clean up:
+
+ cme modify ssh 'Host:-"baz" Host:-"foo* bar*"'
 
 =head1 Methods
 
@@ -1055,7 +1168,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
