@@ -1,17 +1,19 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Node;
-$Config::Model::Node::VERSION = '2.061';
+$Config::Model::Node::VERSION = '2.082';
 use Mouse;
+with "Config::Model::Role::NodeLoader";
 
 use Carp;
+use 5.010;
 
 use Config::Model::Exception;
 use Config::Model::Loader;
@@ -40,6 +42,7 @@ my %legal_properties = (
 
 my $logger     = get_logger("Tree::Node");
 my $fix_logger = get_logger("Anything::Fix");
+my $change_logger = get_logger("ChangeTracker");
 
 # Here are the legal element types
 my %create_sub_for = (
@@ -96,6 +99,14 @@ has needs_save => ( is => 'rw', isa => 'Bool', default => 0 );
 
 has backend_mgr => ( is => 'ro', isa => 'Maybe[Config::Model::BackendMgr]' );
 
+# attribute is defined in Config::Model::Anythin
+sub _backend_support_annotation {
+    my $self = shift;
+    return $self->backend_mgr ? $self->backend_mgr->support_annotation
+        :  $self->parent      ? $self->parent->backend_support_annotation
+        :                       undef ; # no backend at all. test only
+}
+
 sub BUILD {
     my $self = shift;
 
@@ -115,6 +126,8 @@ sub BUILD {
     # get_model returns a cloned data structure
     $self->model( $self->config_model->get_model($class_name) );
 
+    $self->{original_model} = dclone($self->model);
+
     $self->check_properties;
 
     return $self;
@@ -157,7 +170,8 @@ sub create_element {
     my $method = $create_sub_for{ $element_info->{type} };
 
     croak $self->{config_class_name},
-        " error: no create method for element type $element_info->{type}"
+        " error: unknown element type $element_info->{type}, expected ",
+        join(' ', sort keys %create_sub_for)
         unless defined $method;
 
     $self->$method( $element_name, $check );
@@ -167,6 +181,7 @@ sub create_node {
     my ( $self, $element_name, $check ) = @_;
 
     my $element_info = dclone( $self->{model}{element}{$element_name} );
+    my $config_class_name = $element_info->{config_class_name};
 
     Config::Model::Exception::Model->throw(
         error  => "create node '$element_name' error: " . "missing config class name parameter",
@@ -174,7 +189,7 @@ sub create_node {
     ) unless defined $element_info->{config_class_name};
 
     my @args = (
-        config_class_name => $element_info->{config_class_name},
+        config_class_name => $config_class_name,
         instance          => $self->{instance},
         element_name      => $element_name,
         check             => $check,
@@ -182,7 +197,7 @@ sub create_node {
         container         => $self,
     );
 
-    $self->{element}{$element_name} = $self->new(@args);
+    $self->{element}{$element_name} = $self->load_node(@args);
 }
 
 sub create_warped_node {
@@ -245,7 +260,7 @@ sub create_id {
     croak "Undefined id_class for type '$type'"
         unless defined $id_class_hash{$type};
 
-    my $id_class = delete $element_info->{ $type . '_class' }
+    my $id_class = delete $element_info->{class}
         || 'Config::Model::' . $id_class_hash{$type};
 
     if ( not defined *{ $id_class . '::' } ) {
@@ -310,7 +325,11 @@ sub init {
     my $initial_load_backup = $self->instance->initial_load;
     $self->instance->initial_load_start;
 
-    $self->{backend_mgr} ||= Config::Model::BackendMgr->new( node => $self );
+    $self->{backend_mgr} ||= Config::Model::BackendMgr->new(
+        # config_dir spec given by application info
+        config_dir      => $self->instance->config_dir,
+        node => $self,
+    );
 
     if ( defined $model->{read_config} and not $self->skip_read ) {
         $self->read_config_data( check => $self->check );
@@ -357,6 +376,10 @@ sub notify_change {
     my $self = shift;
     my %args = @_;
 
+    $change_logger->debug( "called for ", $self->name, " from ", join( ' ', caller ),
+        " with ", join( ' ', %args ) )
+        if $change_logger->is_debug;
+
     return if $self->instance->initial_load and not $args{really};
 
     $logger->debug( "called while needs_write is ", $self->needs_save, " for ", $self->name )
@@ -364,11 +387,11 @@ sub notify_change {
 
     if ( defined $self->backend_mgr ) {
         $self->needs_save(1);    # will trigger a save in config_file
-        $self->SUPER::notify_change( @_, needs_save => 0 );
+        $self->SUPER::notify_change( %args, needs_save => 0 );
     }
     else {
         # save config_file will be done by a node above
-        $self->SUPER::notify_change( @_, needs_save => 1 );
+        $self->SUPER::notify_change( %args, needs_save => 1 );
     }
 }
 
@@ -622,7 +645,7 @@ sub fetch_element {
     my $check         = $self->_check_check( $args{check} );
     my $accept_hidden = $args{accept_hidden} || 0;
 
-    $self->init($check);
+    $self->init();
 
     my $model = $self->{model};
 
@@ -671,7 +694,7 @@ sub fetch_element {
         # this will also force a rewrite of the file even if no other
         # semantic change was done
         $self->notify_change(
-            msg    => 'dropping deprecated parameter',
+            note   => 'dropping deprecated parameter',
             path   => $self->location . ' ' . $element_name,
             really => 1,
         );
@@ -765,13 +788,27 @@ sub accept_element {
 
     return unless defined $self->{model}{accept};
 
-    foreach my $accept_regexp ( @{ $self->{model}{accept_list} } ) {
-        if ( $name =~ /^$accept_regexp$/ ) {
-            my $acc = $self->{model}{accept}{$accept_regexp};
-            return $self->reset_accepted_element_model( $name, $acc );
-        }
-    }
+    eval {require Text::Levenshtein::Damerau} ;
+    my $has_tld = ! $@ ;
 
+    foreach my $accept_regexp ( @{ $self->{model}{accept_list} } ) {
+        next unless  $name =~ /^$accept_regexp$/;
+        my $element_list = $self->{original_model}{element_list} ;
+
+        if ($has_tld and $element_list and @$element_list) {
+            my $tld = Text::Levenshtein::Damerau->new($name);
+            my $tld_arg = {list => $element_list };
+            my $dist = $tld->dld_best_distance($tld_arg);
+            if ($dist < 3) {
+                my $best = $tld->dld_best_match($tld_arg);
+                warn "Warning: $name is confusingly close to $best (edit distance is $dist). Is there a typo ?\n";
+            }
+
+        }
+
+        my $acc = $self->{model}{accept}{$accept_regexp};
+        return $self->reset_accepted_element_model( $name, $acc );
+    }
     return;
 }
 
@@ -874,16 +911,7 @@ sub load {
 sub load_data {
     my $self = shift;
 
-    if ( @_ == 3 ) {
-
-        # kludge to avoid breaking Config::Model::Itself. Should be removed end of 2013
-        carp "load_data called with even number of parameters. Use named parameters";
-    }
-
-    my %args = @_ == 3
-        ? ( data => $_[0], check => $_[2] )    # should also be removed end of 2013
-        : @_ > 1 ? @_
-        :          ( data => shift );
+    my %args = @_ > 1 ? @_ : ( data => shift );
 
     my $raw_perl_data = delete $args{data};
     my $check         = $self->_check_check( $args{check} );
@@ -1124,13 +1152,11 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -1229,6 +1255,12 @@ element in another configuration class.
 Optional C<string> parameter. This description will be used when
 generating user interfaces.
 
+=item B<class>
+
+Optional C<string> to specify a Perl class to override the default
+implementation (L<Config::Model::Node>).  This Perl Class B<must>
+inherit L<Config::Model::Node>. Use with care.
+
 =item B<element>
 
 Mandatory C<list ref> of elements of the configuration class :
@@ -1290,7 +1322,7 @@ See L<Config::Model::BackendMgr> for details.
 =item B<accept>
 
 Optional list of criteria (i.e. a regular expression to match ) to
-accept unknown parameters. Each criteria will have a list of
+accept unknown elements. Each criteria will have a list of
 specification that will enable C<Config::Model> to create a model
 snippet for the unknown element.
 
@@ -1310,11 +1342,14 @@ Example:
     },
   ]
 
-All C<element> parameters can be used in specifying accepted parameters.
+All C<element> parameters can be used in specifying accepted elements.
+
+If L<Text::Levenshtein::Damerau> is installed, a warning will be issued if an accepted
+element is too close to an existing element.
 
 The parameter C<accept_after> to specify where to insert the accepted element.
-This will not change much the behavior of the tree, but it will help generate
-user interface easier to use.
+This will not change much the behavior of the tree, but will help generate
+more usable user interfaces.
 
 Example:
 
@@ -1388,6 +1423,13 @@ L</"List element">
 The element is a collection of values which are unique in the
 check_list. See L<CheckList>.
 
+=item C<class>
+
+Override the default class for leaf, list and hash elements. The override
+class be inherit L<Config::Model::Value> for leaf element,
+L<Config::Model::HashId> for hash element and
+L<Config::Model::ListId> for list element.
+
 =back
 
 =head2 Node element
@@ -1430,6 +1472,20 @@ C<leaf> (default).
 See L<Config::Model::ListId> and L<Config::Model::AnyId> for more
 details.
 
+=head1 Constructor
+
+The C<new> constructor accepts the following parameters:
+
+=over
+
+=item config_file
+
+Specify configuration file to be used by backend. This parameter may
+override a file declared in the model. Note that this parameter is not
+propagated in children nodes.
+
+=back
+
 =head1 Introspection methods
 
 =head2 name
@@ -1514,6 +1570,12 @@ See L<Config::Model::AnyThing/"root()">
 
 See L<Config::Model::AnyThing/"location()">
 
+=head2 backend_support_annotation
+
+Returns 1 if at least one of the backends attached to self or a parent
+node support to read and write annotations (aka comments) in the
+configuration file.
+
 =head1 Element property management
 
 =head2 get_element_name (  ...  )
@@ -1788,7 +1850,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
