@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ObjTreeScanner;
-$Config::Model::ObjTreeScanner::VERSION = '2.061';
+$Config::Model::ObjTreeScanner::VERSION = '2.082';
 use strict;
 use Config::Model::Exception;
 use Scalar::Util qw/blessed/;
@@ -285,13 +285,11 @@ Config::Model::ObjTreeScanner - Scan config tree and perform call-backs for each
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model ;
- use Log::Log4perl qw(:easy) ;
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new ;
@@ -442,14 +440,14 @@ Optional parameter:
 
 =item fallback
 
-If set to 'node', the scanner will provide default call-back for node
-items. If set to 'leaf', the scanner will set all leaf callback (like
+If set to C<node>, the scanner will provide default call-back for node
+items. If set to C<leaf>, the scanner will set all leaf callback (like
 enum_value_cb ...) to string_value_cb or to the mandatory leaf_cb
 value. "fallback" callback will not override callbacks provided by the
 user.
 
-If set to 'all', equivalent to 'node' and 'leaf'. By default, no
-fallback is provided.
+If set to C<all> , the scanner provides fallbacks for leaf and node. 
+By default, all fallback are provided.
 
 =item auto_vivify
 
@@ -701,7 +699,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
