@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::TermUI;
-$Config::Model::TermUI::VERSION = '2.061';
+$Config::Model::TermUI::VERSION = '2.082';
 use Carp;
 use strict;
 use warnings;
@@ -99,9 +99,10 @@ my %completion_dispatch = (
     desc   => $completion_sub,
     ll     => $completion_sub,
     ls     => $completion_sub,
+    clear  => $completion_sub,
     set    => $leaf_completion_sub,
     delete => $leaf_completion_sub,
-    reset  => $leaf_completion_sub,
+    reset  => $completion_sub,
 );
 
 sub completion {
@@ -117,7 +118,7 @@ sub completion {
         return $completion_dispatch{$main}->( $self, $text, $line, $start );
     }
     elsif ( not $cmd ) {
-        return $self->simple_ui_commands();
+        return grep ( /^$text/, $self->simple_ui_commands() );
     }
 
     return ();
@@ -210,14 +211,12 @@ Config::Model::TermUI - Provides Config::Model UI with Term::ReadLine
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
  use Config::Model::TermUI ;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -377,7 +376,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
