@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ValueComputer;
-$Config::Model::ValueComputer::VERSION = '2.061';
+$Config::Model::ValueComputer::VERSION = '2.082';
 use Mouse;
 use MouseX::StrictConstructor;
 
@@ -466,7 +466,10 @@ sub _value_from_object {
 
     # quote result if asked when calling compute
     my $quote = $need_quote || 0;
-    $my_res = "'$my_res'" if $quote && $my_res;
+    if ($quote && $my_res) {
+        $my_res =~ s/'/\\'/g;
+        $my_res = "'$my_res'";
+    }
 
     return \$my_res;    # So I can return undef ... or a ref to undef
 }
@@ -578,13 +581,11 @@ Config::Model::ValueComputer - Provides configuration value computation
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
  my $model = Config::Model->new;
@@ -1032,7 +1033,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
