@@ -1,15 +1,16 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::WarpedNode;
-$Config::Model::WarpedNode::VERSION = '2.061';
+$Config::Model::WarpedNode::VERSION = '2.082';
 use Mouse;
+with "Config::Model::Role::NodeLoader";
 
 use Carp qw(cluck croak);
 
@@ -143,6 +144,7 @@ sub set_properties {
         Data::Dumper->Dump( [ \%args ], ['set_properties_args'] ) );
 
     my $config_class_name = delete $args{config_class_name};
+    my $node_class = delete $args{class} || 'Config::Model::Node';
 
     my @prop_args = ( qw/property level element/, $self->element_name );
 
@@ -220,7 +222,7 @@ sub create_node {
 
     push @args, index_value => $self->index_value if defined $self->index_value;
 
-    return $self->parent->new(@args);
+    return $self->load_node(@args);
 }
 
 sub clear {
@@ -308,13 +310,11 @@ Config::Model::WarpedNode - Node that change config class properties
 
 =head1 VERSION
 
-version 2.061
+version 2.082
 
 =head1 SYNOPSIS
 
  use Config::Model;
- use Log::Log4perl qw(:easy);
- Log::Log4perl->easy_init($WARN);
 
  my $model = Config::Model->new;
  foreach (qw/X Y/) {
@@ -443,6 +443,11 @@ constructor arguments :
 
   XY  => { config_class_name => ['SlaveY', foo => 'bar' ], },
 
+=item B<class>
+
+Specify a Perl class to implement the above config class. This Perl Class B<must> inherit
+L<Config::Model::Node>.
+
 =back
 
 =head1 Forwarded methods
@@ -552,7 +557,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2005-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
