@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2005-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,6 +9,16 @@
 #
 # test model used by t/*.t
 
+my @backend_config = (
+    read_config => [{
+        backend     => 'yaml',
+        config_dir  => '/yaml/',
+        file        => 'hosts.yml',
+        auto_create => 1,
+        full_dump => 0,
+    }],
+);
+
 [
     {
         name => 'Host',
@@ -24,16 +34,7 @@
     {
         name => 'Hosts',
 
-        read_config => [
-            {
-                backend     => 'yaml',
-                config_dir  => '/yaml/',
-                file        => 'hosts.yml',
-                auto_create => 1,
-                full_dump => 0,
-            },
-        ],
-
+        @backend_config,
         element => [
             record => {
                 type  => 'list',
@@ -43,5 +44,32 @@
                 },
             },
         ]
+    },
+    {
+        name => 'SingleHashElement',
+
+        @backend_config,
+
+        element => [
+            record => {
+                type  => 'hash',
+                index_type => 'string',
+                cargo => {
+                    type              => 'node',
+                    config_class_name => 'Host',
+                },
+            },
+        ]
+    },
+    {
+        name => 'TwoElements',
+        include => 'SingleHashElement',
+        @backend_config,
+        element => [
+            foo => {
+                type => 'leaf',
+                value_type => 'uniline',
+            }
+        ]
     }
 ];
